#ifndef LIB_DRAW_MQH_INCLUDED
#define LIB_DRAW_MQH_INCLUDED
/**********************************************************************************
 * Copyright (C) 2010-2021 Dominik Egert <dominik.egert@freie-netze.de>
 *
 * This file is part of the MQLplus library.
 *
 * MQLplus not be copied and/or distributed without explecit permit.
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  File information
 *  ================
 *
 *  Version: 1.00
 *
 *  Use: lib_draw.mqh include file as part ofg MQLplus
 *
 *
*/



/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* Color functions                                                                                                                                       */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/
namespace colors
{
#define IND_DISPLAY_COLORS 30
#define clrBGShaddow        0x7ECECECE
#define clrBGInverse        0x7FCECECE

// Color split structure
struct _clr_rgb
{
    uchar red;
    uchar green;
    uchar blue;
    uchar alpha;
};

union _clr_transform
{
    _clr_rgb rgb;
    uint     raw;
    color    clr;
    _clr_transform(const color inp = NULL) { clr = inp; };
    void operator=(const color inp) { clr = inp; };
};


// Create custom color
color custom_color(const uchar red, const uchar green, const uchar blue, const uchar alpha = 0xFF)
{
    _clr_transform clr;
    clr.rgb.red     = blue;
    clr.rgb.green   = green;
    clr.rgb.blue    = red;
    clr.rgb.alpha   = alpha;

    // Return color
    return(clr.clr);
}

// Set transparency
color transparent_color(const color color_in, const double transparency)
{
    // Add alpha value
    _clr_transform clr  = color_in;
//    clr.clr         = color_in;
    clr.rgb.alpha       = 255 - (uchar)(255.0 * (transparency / ((transparency > 1.0) ? 100.0 : 1.0)));

    // Return
    return(clr.clr);
}

// Calculate a shadow for a given color
color shaddow_color(const color color_in)
{
    _clr_transform clr;
    clr.clr = color_in;
    
    clr.rgb.red     = (uchar)((clr.rgb.red > 127)    ? clr.rgb.red -   (clr.rgb.red / 2)    : clr.rgb.red +   (clr.rgb.red / 2));
    clr.rgb.green   = (uchar)((clr.rgb.green > 127)  ? clr.rgb.green - (clr.rgb.green / 2)  : clr.rgb.green + (clr.rgb.green / 2));
    clr.rgb.blue    = (uchar)((clr.rgb.blue > 127)   ? clr.rgb.blue -  (clr.rgb.blue / 2)   : clr.rgb.blue +  (clr.rgb.blue / 2));
    clr.rgb.alpha   = 127;
    
    // Return
    return(clr.clr);
}

// Invert a color
color invert_color(const color color_in)
{
    _clr_transform clr;
    clr.clr = color_in;

    if(color_in == clrNONE)
    {
        clr.rgb.red     = (uchar)0x254;
        clr.rgb.green   = (uchar)0x254;
        clr.rgb.blue    = (uchar)0x254;

        // Return
        return(clr.clr);
    }    
    
    clr.rgb.red     = (uchar)((clr.rgb.red      - 255) * -1);
    clr.rgb.green   = (uchar)((clr.rgb.green    - 255) * -1);
    clr.rgb.blue    = (uchar)((clr.rgb.blue     - 255) * -1);

    // Return
    return(clr.clr);
}

// Add color escape sequence to string
string colored_string(const string str_in, const color clr = NULL, const bool bold = false, const bool underline = false, const bool invert = false, const bool background_clr = false)
{
    // Local init
    const static string esc_sequence = CharToString(0x5C) + CharToString(0x1B) + CharToString(0x5B);
    uint clr_code = NULL;

    // Switch color code
    switch(clr)
    {
        case clrBlack:      clr_code = 30 + (10 * background_clr); break;
        case clrRed:        clr_code = 31 + (10 * background_clr); break;
        case clrGreen:      clr_code = 32 + (10 * background_clr); break;
        case clrOrange:     clr_code = 33 + (10 * background_clr); break;
        case clrBlue:       clr_code = 34 + (10 * background_clr); break;
        case clrMagenta:    clr_code = 35 + (10 * background_clr); break;
        case clrCyan:       clr_code = 36 + (10 * background_clr); break;
        case clrWhite:      clr_code = 37 + (10 * background_clr); break;
        default:            clr_code = ColorToARGB(clr);
    }    

    // Return string
    return(StringFormat("%s%u;%um%s%s0m", esc_sequence, (((bold) ? 1 : 21) + ((underline) ? 4 : 24) + ((invert) ? 7 : 27)) * (bold || underline || invert), clr_code, str_in, esc_sequence));
}


// Function to interpolate between two colors
void interpolate_colors(const color color1, const color color2, color& result[], const uchar steps = 0xFF) 
{
    // Local init

        _clr_transform clr1;
        _clr_transform clr2;    
        _clr_transform clr_result;    
        double factor 		= NULL;
		double stepFactor 	= 1.0 / (steps - 1);
		ZeroMemory(clr1);
		ZeroMemory(clr2);
		ZeroMemory(clr_result);
        clr1.clr 			= color1;
        clr2.clr 			= color2;


    // Init result array
    
        ArrayResize(result, steps);
        ZeroMemory(result);


    // Loop color steps

        for(int cnt = 0; cnt < steps; cnt++) 
        {
			// Update factor
			factor = stepFactor * cnt;
			
            // Change color
            clr_result.rgb.red    = (uchar)(NormalizeDouble(MathRound( clr1.rgb.red + factor * (clr2.rgb.red - clr1.rgb.red) ), 0));
            clr_result.rgb.green  = (uchar)(NormalizeDouble(MathRound( clr1.rgb.green + factor * (clr2.rgb.green - clr1.rgb.green) ), 0));
            clr_result.rgb.blue   = (uchar)(NormalizeDouble(MathRound( clr1.rgb.blue + factor * (clr2.rgb.blue - clr1.rgb.blue) ), 0));
        
            // Save result
            result[cnt] = clr_result.clr; 
        }

    // Return
    return;
}

// Function to calculate color gradient
double gradient_color(const int intensity_percent)
{ return(gradient_color((double)(intensity_percent) / 100.0)); }

double gradient_color(const double intensity_factor)
{
    // Check input
    if(intensity_factor == NULL)
    { return(NULL); }

    // Check for second gradient
    if(intensity_factor < NULL)
    { return(color_intensity_index(fabs(intensity_factor), 1.0, 2)); }
    
    // First gradient
    else
    { return(color_intensity_index(intensity_factor)); }

    // Return result
    return(NULL);
}

// Function get color index from gradient
double color_intensity_index(const double value, const double max = 1.0, const int color_scale = 1)
{
    // Calculate
    int idx = (int)NormalizeDouble(fabs(value) * (IND_DISPLAY_COLORS / max), 0);

    // Check result
    if(idx > IND_DISPLAY_COLORS)
    { return((double)IND_DISPLAY_COLORS); }

    // Check scale
    if( (idx == 0x00) 
     || (idx == (int)EMPTY_VALUE) )
    { return((double)(IND_DISPLAY_COLORS * (color_scale - 1))); }

    // Return result
    return((double)idx);
}

// Function rendering string to bitmap based on metadata
bool TextOutColor(const string text, const int text_len, const int char_width, const int txtout_fontsize, const int _x_pos, const int _y_pos, const uint anchor, uint& out_bmp[], const uint width, const uint height, const uint clr, const uint bg_clr, const ENUM_COLOR_FORMAT clr_format)
{
    // Local init
    int         str_pos         = NULL;
    int         mt_split_pos    = NULL;
    int         mt_cmnd         = -1;
    uint        mt_clr          = NULL;
    uint        txt_flag        = FW_LIGHT;
    string      txt_out         = NULL;
    uint        txt_pos         = NULL;
    uint        txtout_clr      = NULL;
    bool        bError          = false;
    bool        inverse         = false;
    string      tmp             = NULL;
    string      metadata[];

    uint        x_pos           = _x_pos;
    uint        y_pos           = _y_pos;
    const int   mt_cnt          = StringSplit(StringSubstr(text, text_len), 0x23, metadata);

    // Split and render text according to metadata
    for(int loop = NULL; (loop < mt_cnt) && !_StopFlag && (!bError); loop++)
    {
        mt_split_pos    = StringFind(metadata[loop], ":", 1);
        str_pos         = (int)StringToInteger(StringSubstr(metadata[loop], 1, mt_split_pos));
        tmp             = StringSubstr(metadata[loop], mt_split_pos + 1, 1);
        mt_cmnd         = (tmp != ";") ? (int)StringToInteger(tmp) : -1;
        tmp             = StringSubstr(metadata[loop], StringFind(metadata[loop], ";", mt_split_pos + 2), StringFind(metadata[loop], "m", mt_split_pos + 3));
        mt_clr          = (uint)StringToInteger(tmp);

        if(str_pos == txt_pos)
        {
            switch(mt_cmnd)
            {
                // Reset
                case 0:     txtout_clr  = clr; txt_flag |= FW_LIGHT; break;
                // Bold on
                case 1:     txt_flag |= FW_HEAVY; break;
                // Underline on
                case 4:     txt_flag |= FONT_UNDERLINE; break;
                // Color inversion on
                case 7:     inverse = true; break;
                // Bold off
                case 21:    txt_flag = txt_flag & (!(FW_HEAVY)); break;
                // Underline off
                case 24:    txt_flag = txt_flag & (!(FONT_UNDERLINE)); break;
                // Color inversion off
                case 27:    inverse = false; break;
            }

            switch(mt_clr)
            {
                case 30:    txtout_clr  = ColorToARGB(clrBlack); break;
                case 31:    txtout_clr  = ColorToARGB(clrRed); break;
                case 32:    txtout_clr  = ColorToARGB(clrGreen); break;
                case 33:    txtout_clr  = ColorToARGB(clrOrange); break;
                case 34:    txtout_clr  = ColorToARGB(clrBlue); break;
                case 35:    txtout_clr  = ColorToARGB(clrMagenta); break;
                case 36:    txtout_clr  = ColorToARGB(clrCyan); break;
                case 37:    txtout_clr  = ColorToARGB(clrWhite); break;
                
                // Background color
                case 40:    
                case 41:
                case 42:
                case 43:
                case 44:
                case 45:
                case 46:
                case 47:

                // Use alpha as distinguish info for background color selection
                default:
                {
                    switch(mt_clr & 0xFF000000)
                    {
                        case 0xFF:  txtout_clr = mt_clr; break;
                        default:
                            // Set background color
                            ArrayFill(out_bmp, 0, 0, 0x00);
                    }
                }
            }
        }
        txt_out = StringSubstr(text, txt_pos, str_pos - (txt_pos * (str_pos > NULL)));
        bError |= (!bError) && !TextSetFont("Lucida Sans Typewriter", txtout_fontsize * -10, txt_flag, 0);
        bError |= (!bError) && !TextOut(txt_out, x_pos, y_pos, TA_LEFT|TA_TOP, out_bmp, width, height, txtout_clr, COLOR_FORMAT_ARGB_RAW);
        x_pos += char_width * StringLen(txt_out);
    }

    // Return
    return(!bError);
}



};
//
// END Colors functions                                                                                                                           */
//*********************************************************************************************************************************************************/








/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* Draw Object database and helper functions                                                                                                             */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/
namespace draw
{

// Initialize type of preset
#ifndef IND_DISPLAY_ONCHART
#define IND_DRAW_MAINWINDOW false
#else 
#define IND_DRAW_MAINWINDOW true
#endif

#define IND_DRAWTYPE_ARROW      0x01
#define IND_DRAWTYPE_VLINE      0x02
#define IND_DRAWTYPE_HLINE      0x03
#define IND_DRAWTYPE_TLINE      0x04
#define IND_DRAWTYPE_RECT       0x05
#define IND_DRAWTYPE_TEXT       0x06
#define IND_DRAWTYPE_TEXTFIELD  0x07
#define IND_DRAWTYPE_EDIT       0x08
#define IND_DRAWTYPE_LABEL      0x09
#define IND_DRAWTYPE_RECT_LABEL 0x0A
#define IND_DRAWTYPE_CONSOLE    0x0B
#define IND_DRAWTYPE_TRIANGLE   0x0C
#define IND_DRAWTYPE_ELLIPSE    0x0D
#define IND_DRAWTYPE_BUTTON     0x0E

#define IND_OBJTYPE_DYNAMIC     0x80

// Object database

struct object_data
{ 
    long    uid;
    uint    x_pos;
    uint    y_pos;
    uint    d_width;
    uint    d_height;
    ulong   flags;
    string  linked_obj;
    string  data[]; 

    object_data() : uid(NULL),
                    x_pos(NULL), 
                    y_pos(NULL), 
                    d_width(NULL), 
                    d_height(NULL), 
                    flags(NULL),
                    linked_obj(NULL) 
    {};
};

static long         obj_counter = 1;
static object_data  obj_data[];

struct object_def
{
    const   long    uid;
            long    chart_id;
            int     window_id;
            int     type;
            uint    px_height;
            uint    px_width;
            uint    brd_style;
            color   clr;
            color   bgclr;
            bool    floating;
            bool    dynamic;
            bool    chart_redraw;
            int     data_idx;
            string  obj_name;

    object_def() :  uid(obj_counter++), 
                    chart_id(NULL), 
                    type(NULL), 
                    px_height(NULL), 
                    px_width(NULL), 
                    brd_style(NULL), 
                    clr(NULL),
                    bgclr(NULL),
                    floating(false), 
                    dynamic(false), 
                    chart_redraw(false), 
                    data_idx(-1), 
                    obj_name(NULL) 
    {};
};

static object_def   obj_list[];



//+------------------------------------------------------------------+
//| Chart object ID database handler functions                       |
//+------------------------------------------------------------------+
const string object_id(const long chart_id, const int _object_type, const long uid = 0x00, const bool del_obj = false)
{ 
    // Local init

        const int object_type = _object_type - (_object_type & IND_OBJTYPE_DYNAMIC);


    // Check for existing

        if((uid < -1) || (uid > NULL))
        {
            const int idx = object_idx(uid);

            if( (idx != -1)
             && (obj_list[idx].type == _object_type) )
            {
                // Delete object from list
                if(del_obj)
                { 
                    // Check for linked objects
                    int chain = idx;
                    int chain_list[];
                    int chain_length = NULL;
                    while( (obj_list[chain].data_idx != -1)
                        && (obj_data[obj_list[chain].data_idx].linked_obj != NULL) 
                        && !_StopFlag )
                    { 
                        chain = object_idx(obj_data[obj_list[chain].data_idx].linked_obj); 
                        ArrayResize(chain_list, chain_length + 1, 25);
                        chain_list[chain_length] = chain;
                        chain_length++;
                    }
                    for(int cnt = chain_length - 1; (cnt > -1) && !_StopFlag; cnt--)
                    {
                        // Delete object
                        ObjectDelete(obj_list[chain_list[cnt]].chart_id, obj_list[chain_list[cnt]].obj_name);
                        (obj_list[chain_list[cnt]].data_idx != -1) && ArrayRemove(obj_data, obj_list[chain_list[cnt]].data_idx, 0x01);
                        ArrayRemove(obj_list, chain_list[cnt], 0x01); 
                    }
                                        
                    // Delete object
                    ObjectDelete(obj_list[idx].chart_id, obj_list[idx].obj_name);
                    (obj_list[idx].data_idx != -1) && ArrayRemove(obj_data, obj_list[idx].data_idx, 0x01);
                    (obj_list[idx].data_idx != -1) && object_update_data_idx();
                    ArrayRemove(obj_list, idx, 0x01); 

                    // Return empty string
                    return(NULL);
                }

                // Return object string
                return(obj_list[idx].obj_name);
            }
        }
        

    // Abort failed delete

        if(del_obj)
        { return(StringFormat("%s_obj_%i_%i", ((_object_type & IND_OBJTYPE_DYNAMIC) == IND_OBJTYPE_DYNAMIC) ? "d" : "p", object_type, uid)); }


    // Create new object

        const int curr_obj_cnt = ArraySize(obj_list);

        if(ArrayResize(obj_list, curr_obj_cnt + 1, 100) < 1)
        { return(NULL); }

        obj_list[curr_obj_cnt].chart_id     = chart_id;
        obj_list[curr_obj_cnt].type         = object_type;
        obj_list[curr_obj_cnt].dynamic      = (_object_type & IND_OBJTYPE_DYNAMIC);
        obj_list[curr_obj_cnt].chart_redraw = obj_list[curr_obj_cnt].dynamic;
        obj_list[curr_obj_cnt].obj_name     = StringFormat("%s_obj_%i_%i", (obj_list[curr_obj_cnt].dynamic) ? "d" : "p", obj_list[curr_obj_cnt].type, obj_list[curr_obj_cnt].uid);         

    // Return object string
    return(obj_list[curr_obj_cnt].obj_name);
}

const long object_id(const string obj_str)
{
    // Extract uid
    int     obj_type    = NULL;
    bool    dynamic     = false;
    return(object_id(obj_str, obj_type, dynamic));
}

const long object_id(const string obj_str, int& object_type, bool& dynamic)
{
    // Check id length
	if(StringLen(obj_str) < 3)
	{ return(NULL); }

    // Extract object type
    dynamic     = (StringSubstr(obj_str, NULL, 1) == "d");
    object_type = (int)StringToInteger(StringSubstr(obj_str, 6, StringFind(obj_str, "_", 6) - 6));
    
    // Return
    return(StringToInteger(StringSubstr(obj_str, StringFind(obj_str, "_", 7) + 1)));
}


const int object_type(const string obj_str)
{
    // Extract uid
    int     obj_type    = NULL;
    bool    dynamic     = false;
    long    uid         = object_id(obj_str, obj_type, dynamic);
    return(obj_type);
}


const int object_idx(const string obj_str)
{
	// Get object UID
	const 	ulong uid = object_id(obj_str);

    // Return
    return(object_idx(uid));
}

const int object_idx(const long uid)
{
	// Local init
	const   int curr_obj_cnt    = ArraySize(obj_list);
			int cnt             = NULL;

	// Search
	for(; (cnt < curr_obj_cnt) && !_StopFlag && (obj_list[cnt].uid != uid); cnt++);

    // Return
	return((cnt < curr_obj_cnt) ? cnt : -1);
}


bool object_update_data_idx()
{
    // Local init
	const   int curr_obj_cnt    = ArraySize(obj_list);
	const   int curr_data_cnt   = ArraySize(obj_data);
            int data_cnt        = NULL;

    // Update by uid
    for(int cnt = NULL; (cnt < curr_obj_cnt) && !_StopFlag; cnt++)
    {
        for(; (data_cnt < curr_data_cnt) && !_StopFlag && (obj_list[cnt].uid != obj_data[data_cnt].uid); data_cnt++);
        obj_list[cnt].data_idx = data_cnt;
        data_cnt = NULL;
    }
    
    // Return
    return(true);
}

bool objects_delete()
{
	// Local init
	const int curr_obj_cnt = ArraySize(obj_list);
	long chart_id = NULL;
    bool err = false;

	// Delete everything
	for(int idx = NULL; (idx < curr_obj_cnt); idx++)
	{ 
	    err |= !ObjectDelete(obj_list[idx].chart_id, obj_list[idx].obj_name); 
	    (chart_id != obj_list[idx].chart_id) && object_redraw(obj_list[idx].chart_id);
	    chart_id = (obj_list[idx].chart_id * (chart_id != obj_list[idx].chart_id)) + (chart_id * (chart_id == obj_list[idx].chart_id));
	}
    object_redraw(chart_id);

    // Free arrays
    if(!err)
    {
        ArrayFree(obj_list);
        ArrayFree(obj_data);
    }
    
    // Return
    return(!err);
}

bool object_redraw(const long chart_id)
{
    ChartRedraw(chart_id);
    return(true);
}


// END Draw Object database and helper functions                                                                                                          */
//*********************************************************************************************************************************************************/







/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* Drawing functions                                                                                                                                     */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/


//+------------------------------------------------------------------+
//| Draw arrows on chart window                                      |
//+------------------------------------------------------------------+
bool arrow_remove(string obj_name)
{ return(arrow(obj_name, LONG_MAX, DBL_MAX)); }

string arrow(   const datetime          tm_idx          = NULL, 
                const double            chart_price     = NULL, 
                const color             objColor        = clrGray, 
                const ENUM_OBJECT       arrow_type      = NULL, 
                const uint              arrow_size      = NULL, 
                const ENUM_LINE_STYLE   style           = NULL, 
                const string            tooltip         = NULL,
                const uint              wingdings_code  = NULL,
                const long              chart_id        = NULL, 
                const int               window_id       = NULL)
{
    string tmp_id = NULL; 
    return((arrow(tmp_id, tm_idx, chart_price, objColor, arrow_type, arrow_size, style, tooltip, wingdings_code, chart_id, window_id)) ? tmp_id : NULL);
}

bool arrow( string&                 obj_name, 
            const datetime          tm_idx          = NULL, 
            const double            chart_price     = NULL, 
            const color             objColor        = NULL, 
            const ENUM_OBJECT       arrow_type      = NULL, 
            const uint              arrow_size      = NULL, 
            const ENUM_LINE_STYLE   style           = (ENUM_LINE_STYLE)EMPTY_VALUE, 
            const string            tooltip         = NULL,
            const uint              wingdings_code  = (uint)EMPTY_VALUE,
            const long              chart_id        = NULL, 
            const int               window_id       = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        uint            obj_anchor  = NULL;
        bool            set_width   = false;
        color           obj_color   = NULL;
        MqlRates        rates[];
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_ARROW) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (tm_idx == LONG_MAX)
         && (chart_price == DBL_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_ARROW, obj_list[obj_idx].uid, true) == NULL); }
        else if( (chart_price == DBL_MAX)
              || (tm_idx == LONG_MAX) )
        { return(false); }
      

    // Determine anchor point 

        if(CopyRates(ChartSymbol((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), ChartPeriod((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), tm_idx, 1, rates))
        {
            if(rates[0].low < chart_price)
            { obj_anchor  = ANCHOR_LEFT_LOWER; }
            else if(rates[0].high > chart_price)
            { obj_anchor  = ANCHOR_LEFT_UPPER; }
            else
            { obj_anchor  = ANCHOR_CENTER; }
        }
        else if(obj_idx == -1)
        {
            switch(arrow_type)
            {
                case OBJ_ARROW_THUMB_UP:
                case OBJ_ARROW_UP:
                case OBJ_ARROW_CHECK:
                case OBJ_ARROW_BUY:
                    obj_anchor  = ANCHOR_LEFT_LOWER;
                    break;
    
                case OBJ_ARROW_THUMB_DOWN:
                case OBJ_ARROW_DOWN:
                case OBJ_ARROW_STOP:
                case OBJ_ARROW_SELL:
                    obj_anchor  = ANCHOR_LEFT_UPPER;
                    break;
    
                case OBJ_ARROW_LEFT_PRICE:
                case OBJ_ARROW_RIGHT_PRICE:
                case OBJ_ARROW:
                default:
                    obj_anchor  = ANCHOR_CENTER;
                    break;
            }
        }


    // Switch color

        if(obj_idx == -1)
        {
            switch(arrow_type)
            {
                case OBJ_ARROW_THUMB_UP:
                case OBJ_ARROW_UP:
                case OBJ_ARROW_CHECK:
                case OBJ_ARROW_BUY:
                    obj_color   = ((objColor == NULL) ? clrGreen : objColor);
                    break;
    
                case OBJ_ARROW_THUMB_DOWN:
                case OBJ_ARROW_DOWN:
                case OBJ_ARROW_STOP:
                case OBJ_ARROW_SELL:
                    obj_color   = ((objColor == NULL) ? clrRed : objColor);
                    break;
    
                case OBJ_ARROW_LEFT_PRICE:
                case OBJ_ARROW_RIGHT_PRICE:
                case OBJ_ARROW:
                default:
                    obj_color = ((objColor == NULL) ? clrGray : objColor);
                    break;
            }
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_ARROW, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw arrow

        // Update existing
        if(obj_idx != -1)
        { 
            ( (tm_idx != NULL) || (chart_price != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 0,                             (tm_idx == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : tm_idx, 
                                                                                            (chart_price == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : chart_price);
            
            if( (objColor != NULL) 
             && (objColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            (arrow_size != NULL) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           arrow_size);
            
            (wingdings_code != (uint)EMPTY_VALUE) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_ARROWCODE,       wingdings_code);
            
            (style != (ENUM_LINE_STYLE)EMPTY_VALUE) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;

            ObjectCreate(chart_id, obj_name, ((arrow_type == NULL) ? OBJ_ARROW : arrow_type), window_id, tm_idx, chart_price);

            ObjectSetInteger(chart_id,  obj_name, OBJPROP_COLOR,         obj_color);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_WIDTH,         ((arrow_size < 1) ? 1 : arrow_size));
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_ANCHOR,        obj_anchor);
            ((arrow_type == NULL) || (arrow_type == OBJ_ARROW)) &&
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_ARROWCODE,     (((wingdings_code == NULL) || (wingdings_code == (uint)EMPTY_VALUE)) ? 161 : wingdings_code));
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_STYLE,         (style == (ENUM_LINE_STYLE)EMPTY_VALUE) ? STYLE_DOT : style);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TOOLTIP,       (tooltip == NULL) ? "\n" : tooltip);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_HIDDEN,        true);
        }

    // Return
    return(obj_name != NULL);
      
}



//+------------------------------------------------------------------+
//| Draw horizontal line on chart window                             |
//+------------------------------------------------------------------+
bool horizontalline_remove(string obj_name)
{ return(horizontalline(obj_name, DBL_MAX, NULL, NULL, NULL, NULL, NULL)); }

string horizontalline(  const double    chart_price     = NULL, 
                        const color     objColor        = clrGray, 
                        const uint      solid           = 0x05,
                        const string    tooltip         = NULL,
                        const long      chart_id        = NULL, 
                        const int       window_id       = NULL)
{
    string tmp_id = NULL; 
    return((horizontalline(tmp_id, chart_price, objColor, solid, tooltip, chart_id, window_id)) ? tmp_id : NULL);
}

bool horizontalline(    string&         obj_name, 
                        const double    chart_price     = NULL, 
                        const color     objColor        = clrGray, 
                        const uint      solid           = 0x05, 
                        const string    tooltip         = NULL,
                        const long      chart_id        = NULL, 
                        const int       window_id       = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style       = STYLE_DOT;
        bool            err         = false;
        bool            set_width   = false;
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_HLINE) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (chart_price == DBL_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_HLINE, obj_list[obj_idx].uid, true) == NULL); }
        else if(chart_price == DBL_MAX)
        { return(false); }
      
      
    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
                
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_HLINE, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);
        err |= (obj_name == NULL);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw line

        // Update existing
        if(obj_idx != -1)
        { 
            (chart_price != NULL) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 0, NULL, chart_price);

            if( (objColor != NULL) 
             && (objColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           solid - 5);

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else if(!err)
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;

            err |= !ObjectCreate(chart_id, obj_name, OBJ_HLINE, window_id, NULL, chart_price);
            
            err |= !ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            err |= !ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            err |= !ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       ((tooltip == NULL) ? "\n" : tooltip));
            err |= !ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            err |= !ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            err |= !ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            err |= !ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            err |= !(!set_width || ((set_width) && 
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         solid - 5)));
        }


    // Return
    return((obj_name != NULL) && !err);
}



//+------------------------------------------------------------------+
//| Draw vertical line on chart window                               |
//+------------------------------------------------------------------+
bool verticalline_remove(string obj_name) 
{ return(verticalline(obj_name, (datetime)LONG_MAX, NULL, NULL, NULL, false, NULL, NULL)); }

string verticalline(const datetime  tm_idx      = NULL, 
                    const color     objColor    = clrGray, 
                    const uint      solid       = 0x05, 
                    const string    tooltip     = NULL, 
                    const bool      ray         = false, 
					const long      chart_id        = NULL, 
                    const int       window_id   = NULL)
{
    string tmp_id = NULL; 
    return((verticalline(tmp_id, tm_idx, objColor, solid, tooltip, ray, chart_id, window_id)) ? tmp_id : NULL);
}

bool verticalline(  string&         obj_name, 
                    const datetime  tm_idx      = NULL, 
                    const color     objColor    = clrGray, 
                    const uint      solid       = 0x05, 
                    const string    tooltip     = NULL, 
                    const bool      ray         = false, 
					const long      chart_id    = NULL, 
                    const int       window_id   = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style       = STYLE_DOT;
        bool            set_width   = false;
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_VLINE) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (tm_idx == LONG_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_VLINE, obj_list[obj_idx].uid, true) == NULL); }
        else if(tm_idx == LONG_MAX)
        { return(false); }

      
    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_VLINE, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw verticalline

        // Update existing
        if(obj_idx != -1)
        { 
            (tm_idx != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME,            tm_idx);
            
            if( (objColor != NULL) 
             && (objColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           solid - 5);

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));

            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_RAY,             ray);
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;

            ObjectCreate(chart_id, obj_name, OBJ_VLINE, window_id, tm_idx, NULL);
            
            ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_RAY,           ray);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TOOLTIP,      ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            (set_width) && 
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         solid - 5);
        }


    // Return
    return(obj_name != NULL);
}


//+------------------------------------------------------------------+
//| Draw trend line on chart window                                  |
//+------------------------------------------------------------------+
bool trendline_remove(string obj_name)
{ return(trendline(obj_name, (datetime)LONG_MAX, (datetime)LONG_MAX, DBL_MAX, DBL_MAX, NULL, NULL, NULL, (int)EMPTY_VALUE, false, EMPTY_VALUE, NULL, NULL)); } 

string trendline(   const datetime      left        = NULL, 
                    const datetime      right       = NULL, 
                    const double        _high       = NULL, 
                    const double        _low        = NULL, 
                    const color         objColor    = clrGray, 
                    const uint          solid       = 0x05, 
                    const string        tooltip     = NULL, 
                    const int           ray         = (int)EMPTY_VALUE, 
                    const bool          arrow       = false, 
                    const double        angle       = EMPTY_VALUE, 
					const long      	chart_id   	= NULL, 
                    const int           window_id   = NULL)
{
    string tmp_id = NULL; 
    return((trendline(tmp_id, left, right, _high, _low, objColor, solid, tooltip, ray, arrow, angle, chart_id, window_id)) ? tmp_id : NULL);
}

bool trendline( string&             obj_name, 
                const datetime      left        = NULL, 
                const datetime      right       = NULL, 
                const double        _high       = NULL, 
                const double        _low        = NULL, 
                const color         objColor    = clrGray, 
                const uint          solid       = 0x05, 
                const string        tooltip     = NULL, 
                const int           ray         = (int)EMPTY_VALUE, 
                const bool          arrow       = false, 
                const double        angle       = EMPTY_VALUE, 
				const long      	chart_id   	= NULL, 
                const int           window_id   = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style       = STYLE_DOT;
        bool            set_width   = false;
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_TLINE) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (left == LONG_MAX)
         && (right == LONG_MAX)
         && (_high == DBL_MAX)
         && (_low == DBL_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_TLINE, obj_list[obj_idx].uid, true) == NULL); }
        else if( (left == LONG_MAX)
              || (right == LONG_MAX)
              || (_high == DBL_MAX)
              || (_low == DBL_MAX) )
        { return(false); }
      
      
    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_TLINE, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw trendline

        // Update existing
        if(obj_idx != -1)
        {
            ( (left != NULL) || (_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 0,                             (left == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : left, 
                                                                                            (_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : _high);

            ( (right != NULL) || (_low != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 1,                             (right == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 1) : right, 
                                                                                            (_low == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 1) : _low);

            if( (left != NULL) 
             || (_high != NULL)
             || (right != NULL) 
             || (_low != NULL) )
            {
                datetime t_l    = (left == NULL)    ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : left;
                datetime t_r    = (right == NULL)   ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 1) : right;
                double p_l      = (_high == NULL)   ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : _high;
                double p_r      = (_low == NULL)    ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 1) : _low;
                obj_list[obj_idx].px_width                                                = (uint)(MathMax(t_l, t_r) - MathMin(t_l, t_r));
                obj_list[obj_idx].px_height                                               = (uint)(MathMax(p_l, p_r) - MathMin(p_l, p_r));
            }
            
            if( (objColor != NULL) 
             && (objColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           solid - 5);

            (ray != (int)EMPTY_VALUE) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_RAY_LEFT,        (ray > 1) || (ray < NULL));

            (ray != (int)EMPTY_VALUE) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_RAY_RIGHT,       (ray > NULL) || (ray < -1));

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));

            (angle != EMPTY_VALUE) &&
            ObjectSetDouble(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_ANGLE,           angle);
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = (uint)(MathMax(left, right) - MathMin(left, right));
            obj_list[new_idx].px_height                 = (uint)(MathMax(_high, _low) - MathMin(_high, _low));
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;

            (angle == EMPTY_VALUE) && (!arrow) &&
            ObjectCreate(chart_id, obj_name, OBJ_TREND, window_id, MathMin(left, right), MathMax(_high, _low), MathMax(left, right), MathMin(_high, _low));
            (angle == EMPTY_VALUE) && (arrow) &&
            ObjectCreate(chart_id, obj_name, OBJ_ARROWED_LINE, window_id, MathMin(left, right), MathMax(_high, _low), MathMax(left, right), MathMin(_high, _low));
            (angle != EMPTY_VALUE) && 
            ObjectCreate(chart_id, obj_name, OBJ_TRENDBYANGLE, window_id, MathMin(left, right), MathMax(_high, _low), MathMax(left, right), MathMin(_high, _low));
            
            ObjectSetDouble(chart_id,  obj_name, OBJPROP_ANGLE,         (angle == EMPTY_VALUE) ? NULL : angle);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_RAY_LEFT,      ((ray > 1) || (ray < NULL)) && (ray != (int)EMPTY_VALUE));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_RAY_RIGHT,     ((ray > NULL) || (ray < -1)) && (ray != (int)EMPTY_VALUE));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            (set_width) &&
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         solid - 5);
        }

    // Return
    return(obj_name != NULL);
}



//+------------------------------------------------------------------+
//| Draw triangle on chart window                                    |
//+------------------------------------------------------------------+
bool triangle_remove(string obj_name)
{ return(triangle(obj_name, (datetime)LONG_MAX, DBL_MAX, (datetime)LONG_MAX, DBL_MAX, (datetime)LONG_MAX, DBL_MAX, NULL, NULL, false, NULL, NULL, NULL)); } 

string triangle(    const datetime      left, 
                    const double        left_high, 
                    const datetime      top, 
                    const double        top_high, 
                    const datetime      right, 
                    const double        right_high, 
                    const color         objColor    = clrGray, 
                    const uint          solid       = 0x05, 
                    const bool          fill        = false, 
                    const string        tooltip     = NULL, 
					const long      	chart_id   	= NULL, 
                    const int           window_id   = NULL)
{
    string tmp_id = NULL; 
    return((triangle(tmp_id, left, left_high, top, top_high, right, right_high, objColor, solid, fill, tooltip, chart_id, window_id)) ? tmp_id : NULL);
}

bool triangle(  string&             obj_name, 
                const datetime      left, 
                const double        left_high, 
                const datetime      top, 
                const double        top_high, 
                const datetime      right, 
                const double        right_high, 
                const color         objColor    = clrGray, 
                const uint          solid       = 0x05, 
                const bool          fill        = false, 
                const string        tooltip     = NULL, 
				const long      	chart_id   	= NULL, 
                const int           window_id   = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style       = STYLE_DOT;
        bool            set_width   = false;
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_TRIANGLE) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (left == LONG_MAX)
         && (right == LONG_MAX)
         && (top == LONG_MAX)
         && (left_high == DBL_MAX)
         && (right_high == DBL_MAX)
         && (top_high == DBL_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_TRIANGLE, obj_list[obj_idx].uid, true) == NULL); }
        else if( (left == LONG_MAX)
              || (right == LONG_MAX)
              || (top == LONG_MAX)
              || (left_high == DBL_MAX)
              || (right_high == DBL_MAX)
              || (top_high == DBL_MAX) )
        { return(false); }


    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_TRIANGLE, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw triangle

        // Update existing
        if(obj_idx != -1)
        {
            ( (left != NULL) || (left_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 0,                             (left == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : left, 
                                                                                            (left_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : left_high);

            ( (top != NULL) || (top_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 1,                             (top == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 1) : top, 
                                                                                            (top_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 1) : top_high);
            
            ( (right != NULL) || (right_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 2,                             (right == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 2) : right, 
                                                                                            (right_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 2) : right_high);

            if( (left != NULL) 
             || (left_high != NULL)
             || (right != NULL) 
             || (right_high != NULL) )
            {
                datetime t_l    = (left == NULL)        ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : left;
                datetime t_t    = (top == NULL)         ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 1) : top;
                datetime t_r    = (right == NULL)       ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 2) : right;
                double p_l      = (left_high == NULL)   ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : left_high;
                double p_t      = (top_high == NULL)    ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 1) : top_high;
                double p_r      = (right_high == NULL)  ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 2) : right_high;
                obj_list[obj_idx].px_width                                                = (uint)(MathMax(MathMax(t_l, t_t), t_r) - MathMin(MathMin(t_l, t_t), t_r));
                obj_list[obj_idx].px_height                                               = (uint)(MathMax(MathMax(p_l, p_t), p_r) - MathMin(MathMin(p_l, p_t), p_r));
            }
            
            if(objColor != NULL)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            if((obj_list[obj_idx].bgclr != clrNONE) != fill)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_FILL,        fill);
                obj_list[obj_idx].bgclr                                                   = (fill) ? objColor : clrNONE;
            }
            
            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           ((solid - 5) * set_width) + (!set_width));

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) && 
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = (uint)(MathMax(MathMax(left, top), right) - MathMin(MathMin(left, top), right));
            obj_list[new_idx].px_height                 = (uint)(MathMax(MathMax(left_high, top_high), right_high) - MathMin(MathMin(left_high, top_high), right_high));
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;
            obj_list[new_idx].bgclr                     = (fill) ? objColor : clrNONE;

            ObjectCreate(chart_id, obj_name, OBJ_TRIANGLE, window_id, left, left_high, top, top_high, right, right_high);
            
            ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_FILL,          fill);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            (set_width) &&
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         solid - 5);
        }

    // Return
    return(obj_name != NULL);
}



//+------------------------------------------------------------------+
//| Draw ellipse on chart window                                     |
//+------------------------------------------------------------------+
bool ellipse_remove(string obj_name)
{ return(ellipse(obj_name, (datetime)LONG_MAX, DBL_MAX, (datetime)LONG_MAX, DBL_MAX, (datetime)LONG_MAX, DBL_MAX, NULL, NULL, false, NULL, NULL, NULL)); } 

string ellipse( const datetime      left, 
                const double        left_high, 
                const datetime      top, 
                const double        top_high, 
                const datetime      right, 
                const double        right_high, 
                const color         objColor    = clrGray, 
                const uint          solid       = 0x05, 
                const bool          fill        = false, 
                const string        tooltip     = NULL, 
				const long      	chart_id   	= NULL, 
                const int           window_id   = NULL)
{
    string tmp_id = NULL; 
    return((ellipse(tmp_id, left, left_high, top, top_high, right, right_high, objColor, solid, fill, tooltip, chart_id, window_id)) ? tmp_id : NULL);
}

bool ellipse(   string&             obj_name, 
                const datetime      left, 
                const double        left_high, 
                const datetime      top, 
                const double        top_high, 
                const datetime      right, 
                const double        right_high, 
                const color         objColor    = clrGray, 
                const uint          solid       = 0x05, 
                const bool          fill        = false, 
                const string        tooltip     = NULL, 
				const long      	chart_id   	= NULL, 
                const int           window_id   = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style       = STYLE_DOT;
        bool            set_width   = false;
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_ELLIPSE) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (left == LONG_MAX)
         && (right == LONG_MAX)
         && (top == LONG_MAX)
         && (left_high == DBL_MAX)
         && (right_high == DBL_MAX)
         && (top_high == DBL_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_ELLIPSE, obj_list[obj_idx].uid, true) == NULL); }
        else if( (left == LONG_MAX)
              || (right == LONG_MAX)
              || (top == LONG_MAX)
              || (left_high == DBL_MAX)
              || (right_high == DBL_MAX)
              || (top_high == DBL_MAX) )
        { return(false); }


    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_ELLIPSE, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw triangle

        // Update existing
        if(obj_idx != -1)
        {
            ( (left != NULL) || (left_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 0,                             (left == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : left, 
                                                                                            (left_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : left_high);

            ( (top != NULL) || (top_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 1,                             (top == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 1) : top, 
                                                                                            (top_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 1) : top_high);
            
            ( (right != NULL) || (right_high != NULL) ) &&
            ObjectMove(obj_list[obj_idx].chart_id, obj_name, 2,                             (right == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 2) : right, 
                                                                                            (right_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 2) : right_high);

            if( (left != NULL) 
             || (left_high != NULL)
             || (right != NULL) 
             || (right_high != NULL) )
            {
                datetime t_l    = (left == NULL)        ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 0) : left;
                datetime t_t    = (top == NULL)         ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 1) : top;
                datetime t_r    = (right == NULL)       ? (datetime)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, 2) : right;
                double p_l      = (left_high == NULL)   ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 0) : left_high;
                double p_t      = (top_high == NULL)    ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 1) : top_high;
                double p_r      = (right_high == NULL)  ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, 2) : right_high;
                obj_list[obj_idx].px_width                                                = (uint)(MathMax(MathMax(t_l, t_t), t_r) - MathMin(MathMin(t_l, t_t), t_r));
                obj_list[obj_idx].px_height                                               = (uint)(MathMax(MathMax(p_l, p_t), p_r) - MathMin(MathMin(p_l, p_t), p_r));
            }
            
            if(objColor != NULL)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            if((obj_list[obj_idx].bgclr != clrNONE) != fill)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_FILL,        fill);
                obj_list[obj_idx].bgclr                                                   = (fill) ? objColor : clrNONE;
            }
            
            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           ((solid - 5) * set_width) + (!set_width));

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) && 
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = (uint)(MathMax(MathMax(left, top), right) - MathMin(MathMin(left, top), right));
            obj_list[new_idx].px_height                 = (uint)(MathMax(MathMax(left_high, top_high), right_high) - MathMin(MathMin(left_high, top_high), right_high));
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;
            obj_list[new_idx].bgclr                     = (fill) ? objColor : clrNONE;

            ObjectCreate(chart_id, obj_name, OBJ_ELLIPSE, window_id, left, left_high, top, top_high, right, right_high);
            
            ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_FILL,          fill);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            (set_width) &&
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         solid - 5);
        }

    // Return
    return(obj_name != NULL);
}



//+------------------------------------------------------------------+
//| Draw rectangle on chart window                                   |
//+------------------------------------------------------------------+
bool rectangle_remove(string obj_name)
{
    switch(object_type(obj_name))
    {
        case IND_DRAWTYPE_RECT:         return(rectangle(obj_name, (datetime)LONG_MAX, (datetime)LONG_MAX, DBL_MAX, DBL_MAX, NULL, NULL, false, NULL, NULL, NULL));
        case IND_DRAWTYPE_RECT_LABEL:   return(rectangle(obj_name, UINT_MAX, UINT_MAX, UINT_MAX, UINT_MAX, NULL, false, NULL, NULL, NULL));
    }
    return(false);
} 

string rectangle(   const datetime      left, 
                    const datetime      right, 
                    const double        _high, 
                    const double        _low, 
                    const color         objColor    = clrGray, 
                    const uint          solid       = 0x05, 
                    const bool          fill        = false, 
                    const string        tooltip     = NULL, 
					const long      	chart_id   	= NULL, 
                    const int           window_id   = NULL)
{
    string tmp_id = NULL; 
    return((rectangle(tmp_id, left, right, _high, _low, objColor, solid, fill, tooltip, chart_id, window_id)) ? tmp_id : NULL);
}

bool rectangle( string&             obj_name, 
                const datetime      left, 
                const datetime      right, 
                const double        _high, 
                const double        _low, 
                const color         objColor    = clrGray, 
                const uint          solid       = 0x05, 
                const bool          fill        = false, 
                const string        tooltip     = NULL, 
				const long      	chart_id   	= NULL, 
                const int           window_id   = NULL)
{
    // Local init
    
        const int       obj_idx     = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style       = STYLE_DOT;
        bool            set_width   = false;
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_RECT) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (left == LONG_MAX)
         && (right == LONG_MAX)
         && (_high == DBL_MAX)
         && (_low == DBL_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_RECT, obj_list[obj_idx].uid, true) == NULL); }
        else if( (left == LONG_MAX)
              || (right == LONG_MAX)
              || (_high == DBL_MAX)
              || (_low == DBL_MAX) )
        { return(false); }
      
      
    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_RECT, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw trend line

        // Update existing
        if(obj_idx != -1)
        {
            if( (left != NULL) 
             || (_high != NULL) )
            {
                ObjectMove(obj_list[obj_idx].chart_id, obj_name, 0,                         (left == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, NULL) : left, 
                                                                                            (_high == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, NULL) : _high);
                
                obj_list[obj_idx].px_width                                                = (uint)fabs(right - left);
                obj_list[obj_idx].px_height                                               = (uint)fabs(_high - _low);
            }

            if( (right != NULL) 
             || (_low != NULL) )
            {
                ObjectMove(obj_list[obj_idx].chart_id, obj_name, 1,                         (right == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, NULL) : right, 
                                                                                            (_low == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE, NULL) : _low);
                obj_list[obj_idx].px_width                                                = (uint)fabs(right - left);
                obj_list[obj_idx].px_height                                               = (uint)fabs(_high - _low);
            }
            
            if(objColor != NULL)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,       objColor);
                obj_list[obj_idx].clr                                                     = objColor;
            }
            
            if((obj_list[obj_idx].bgclr != clrNONE) != fill)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_FILL,        fill);
                obj_list[obj_idx].bgclr                                                   = (fill) ? objColor : clrNONE;
            }
            
            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,           ((solid - 5) * set_width) + (!set_width));

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,           style);

            (tooltip != NULL) && 
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = (uint)fabs(right - left);
            obj_list[new_idx].px_height                 = (uint)fabs(_high - _low);
            obj_list[new_idx].floating                  = true;
            obj_list[new_idx].clr                       = objColor;
            obj_list[new_idx].bgclr                     = (fill) ? objColor : clrNONE;

            ObjectCreate(chart_id, obj_name, OBJ_RECTANGLE, window_id, left, _high, right, _low);
            
            ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_FILL,          fill);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            (set_width) &&
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         solid - 5);
        }

    // Return
    return(obj_name != NULL);
}



//+------------------------------------------------------------------+
//| Draw rectangle on chart window                                   |
//+------------------------------------------------------------------+
string rectangle(   string              obj_name, 
                    const color         objColor    = clrGray, 
                    const uint          solid       = 0x05)
{ return((rectangle(obj_name, (uint)NULL, (uint)NULL, (uint)NULL, (uint)NULL, objColor, solid, false, NULL, NULL, NULL)) ? obj_name : NULL); }

string rectangle(   const uint          x_left      = NULL, 
                    const uint          y_left      = NULL, 
                    const uint          x_right     = NULL, 
                    const uint          y_right     = NULL, 
                    const color         objColor    = clrGray, 
                    const uint          solid       = 0x05, 
                    const bool          fill        = false, 
                    const string        tooltip     = NULL, 
					const long      	chart_id   	= NULL, 
                    const int           window_id   = NULL)
{
    string tmp_id = NULL; 
    return((rectangle(tmp_id, x_left, y_left, x_right, y_right, objColor, solid, fill, tooltip, chart_id, window_id)) ? tmp_id : NULL);
}

bool rectangle( string&             obj_name, 
                const uint          x_left      = NULL, 
                const uint          y_left      = NULL, 
                const uint          x_right     = NULL, 
                const uint          y_right     = NULL, 
                const color         objColor    = clrGray, 
                const uint          solid       = 0x05, 
                const bool          fill        = false, 
                const string        tooltip     = NULL, 
				const long      	chart_id   	= NULL, 
                const int           window_id   = NULL)
{
    // Local init
    
        const int       obj_idx         = draw::object_idx(obj_name);
        ENUM_LINE_STYLE style           = STYLE_DOT;
        uint            _x_left         = x_left;
        uint            _y_left         = y_left;
        uint            _x_right        = x_right;
        uint            _y_right        = y_right;
        uint            _solid          = solid;
        bool            _fill           = fill;
        bool            set_width       = false;
        bool            obj_textfield   = false;
                

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_RECT_LABEL)
         && (object_type(obj_name) != IND_DRAWTYPE_TEXTFIELD) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (x_left == UINT_MAX)
         && (y_left == UINT_MAX)
         && (x_right == UINT_MAX)
         && (y_right == UINT_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_RECT_LABEL, obj_list[obj_idx].uid, true) == NULL); }
        else if( (x_left == UINT_MAX)
              || (y_left == UINT_MAX)
              || (x_right == UINT_MAX)
              || (y_right == UINT_MAX) )
        { return(false); }
      

    // Check for textfield connection
    
        if( (obj_idx != -1)
         && (object_type(obj_name) == IND_DRAWTYPE_TEXTFIELD) )
        {
            obj_textfield   = true;
            _x_left         = (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XDISTANCE);
            _y_left         = (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YDISTANCE);
            _x_right        = (uint)(ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE) + _x_left);
            _y_right        = (uint)(ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YSIZE) + _y_left + ((obj_data[obj_list[obj_idx].data_idx].linked_obj != NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_data[obj_list[obj_idx].data_idx].linked_obj, OBJPROP_YSIZE) : NULL));
            _fill           = false;
            _solid          = 8;
            set_width       = true;
        }

      
    // Type of line

        switch(solid)
        {
            case 0x00:
            case 0x01:
                style = STYLE_DOT;
                break;

            case 0x02:
                style = STYLE_DASHDOTDOT;
                break;

            case 0x03:
                style = STYLE_DASHDOT;
                break;
            
            case 0x04:
                style = STYLE_DASH;
                break;
            
            case 0x05:
                style = STYLE_SOLID;
                break;
            default:
                style = STYLE_SOLID;
                set_width = true;
        }


    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_RECT_LABEL, ((obj_idx == -1) || obj_textfield) ? NULL : obj_list[obj_idx].uid);
        if(obj_textfield) 
        { 
            const int add_data = ArraySize(obj_data);
            ArrayResize(obj_data, add_data + 1);
            obj_data[add_data].linked_obj               = obj_list[obj_idx].obj_name; 
            obj_list[object_idx(obj_name)].data_idx     = add_data;
            obj_list[object_idx(obj_name)].chart_id     = obj_list[obj_idx].chart_id;
            obj_list[object_idx(obj_name)].window_id    = obj_list[obj_idx].window_id;
        }



    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw trend line

        // Update existing
        if( (obj_idx != -1)
         && (!obj_textfield) )
        {
            (_x_left != NULL) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XDISTANCE,                       _x_left);
            
            (_y_left != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YDISTANCE,                       _y_left);

            if(_x_right != NULL)
            {
                _x_left = (_x_left == NULL) ? (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XDISTANCE) : _x_left;
                _solid  = (_solid == NULL) ? (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH) + 5 : _solid;
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE,                       _x_right - _x_left + ((_solid - 5) * (_solid > 5) * 2 * obj_textfield));
            }
            
            if(_y_right != NULL)
            {
                _y_left = (_y_left == NULL) ? (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YDISTANCE) : _y_left;
                _solid  = (_solid == NULL) ? (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH) + 5 : _solid;            
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YSIZE,                       _y_right - _y_left + ((_solid - 5) * (_solid > 5) * 2 * obj_textfield));
            }
            
            if( (objColor != NULL) 
             && (objColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,                       objColor);
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_BORDER_COLOR,                objColor);
                obj_list[obj_idx].clr                                                                     = objColor;
            }

            if( (objColor != NULL)
             && ((obj_list[obj_idx].bgclr != clrNONE) != fill) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_BGCOLOR,                     objColor * fill); 
                obj_list[obj_idx].bgclr                                                                   = objColor * fill;
            }

            (set_width) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_WIDTH,                           ((solid - 5) * set_width) + (!set_width));

            (solid != NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STYLE,                           style);

            (tooltip != NULL) && 
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,                         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const int new_idx = object_idx(obj_name);
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = _x_right - _x_left + ((_solid - 5) * set_width * 2 * obj_textfield);
            obj_list[new_idx].px_height                 = _y_right - _y_left + ((_solid - 5) * set_width * 2 * obj_textfield);
            obj_list[new_idx].clr                       = objColor;
            obj_list[new_idx].bgclr                     = (_fill) ? objColor : clrNONE;

            ObjectCreate(chart_id, obj_name, OBJ_RECTANGLE_LABEL, window_id, NULL, NULL);

            ObjectSetInteger(chart_id, obj_name, OBJPROP_XDISTANCE,     _x_left - ((_solid - 5) * set_width));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_YDISTANCE,     _y_left - ((_solid - 5) * set_width));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_XSIZE,         _x_right - _x_left + ((_solid - 5) * set_width * 2 * obj_textfield));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_YSIZE,         _y_right - _y_left + ((_solid - 5) * set_width * 2 * obj_textfield));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         (objColor == clrBGShaddow) ? (colors::shaddow_color(colors::invert_color((color)ChartGetInteger(chart_id, CHART_COLOR_BACKGROUND, NULL)))) : objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BORDER_COLOR,  objColor);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_STYLE,         style);
            ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BGCOLOR,       (_fill) ? objColor : clrNONE);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BORDER_TYPE,   BORDER_FLAT);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          true);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
            ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
            (set_width) &&
            ObjectSetInteger(chart_id, obj_name, OBJPROP_WIDTH,         _solid - 5);
        }

    // Return
    return(obj_name != NULL);
}



//+------------------------------------------------------------------+
//| Draw text label on chart window                                  |
//+------------------------------------------------------------------+
bool textlabel_remove(string obj_name)
{
    switch(object_type(obj_name))
    {
        case IND_DRAWTYPE_TEXT:     return(textlabel(obj_name, DBL_MAX, (datetime)LONG_MAX, NULL, NULL, NULL, NULL, NULL, EMPTY_VALUE, NULL, NULL));
        case IND_DRAWTYPE_LABEL:    return(textlabel(obj_name, UINT_MAX, UINT_MAX, NULL, NULL, NULL, NULL, NULL, EMPTY_VALUE, NULL, NULL));
    }
    return(false);
} 

string textlabel(   const double        chart_price     = NULL, 
                    const datetime      tm_idx          = NULL, 
                    const string        text            = NULL, 
                    const string        tooltip         = NULL, 
                    const uint          font_size       = 8, 
                    const color         txtColor        = clrWhite, 
                    const color         bgColor         = clrNONE, 
                    const double        angle           = EMPTY_VALUE, 
					const long      	chart_id   		= NULL, 
                    const int           window_id       = NULL)
{
    string tmp_id = NULL; 
    return((textlabel(tmp_id, chart_price, tm_idx, text, tooltip, font_size, txtColor, bgColor, angle, chart_id, window_id)) ? tmp_id : NULL);
}

bool textlabel( string&             obj_name, 
                const double        chart_price     = NULL, 
                const datetime      tm_idx          = NULL, 
                const string        text            = NULL, 
                const string        tooltip         = NULL, 
                const uint          font_size       = 8, 
                const color         txtColor        = clrWhite, 
                const color         bgColor         = clrNONE, 
                const double        angle           = EMPTY_VALUE, 
				const long      	chart_id   		= NULL, 
                const int           window_id       = NULL)
{
    // Local init
    
        static  uint    txt_height  = NULL;
        static  uint    txt_width   = NULL;
        static  uint    _font_size  = NULL;

        if( (_font_size != font_size) 
         && (font_size != NULL) )
         {
            TextSetFont("Lucida Sans Typewriter", font_size * -10, FW_THIN, 0);
            TextGetSize("A", txt_width, txt_height);
        }

        const   static  bool    is_height   = TextGetSize("A", txt_width, txt_height);
        const           int     obj_idx     = draw::object_idx(obj_name);
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_TEXT) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (chart_price == DBL_MAX)
         && (tm_idx == LONG_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_TEXT, obj_list[obj_idx].uid, true) == NULL); }
        else if( (chart_price == DBL_MAX)
              || (tm_idx == LONG_MAX) )
        { return(false); }

      
    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_TEXT, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw trend line

        // Update existing
        if(obj_idx != -1)
        {
            ((chart_price > NULL) || (tm_idx > NULL)) &&
            ObjectMove(obj_list[object_idx(obj_name)].chart_id, obj_name, 0, 
                        (tm_idx == NULL) ? ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TIME, NULL) : tm_idx, 
                        (chart_price == NULL) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_PRICE) : chart_price);

            if( (font_size != _font_size) 
             || (text != NULL) 
             || (angle != EMPTY_VALUE))
            { 
                const uint      _width = StringLen((text != NULL) ? text : ObjectGetString(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TEXT)) * txt_width;
                const double    _angle = ((angle == EMPTY_VALUE) || (angle == NULL)) ? NULL : ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_ANGLE);

                (font_size != _font_size) &&
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_FONTSIZE,                    font_size); 

                ObjectSetString(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TEXT,                         ((text == NULL) ? "\n" : text)); 
                ObjectSetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_ANGLE,                        _angle);

                obj_list[obj_idx].px_width                                                                = ((_angle != NULL) && (_angle != EMPTY_VALUE)) ? (uint)(cos(fabs(_angle)) * _width) : _width;
                obj_list[obj_idx].px_height                                                               = ((_angle != NULL) && (_angle != EMPTY_VALUE)) ? (uint)(sin(fabs(_angle)) * _width) + txt_height : txt_height;
            }

            if( (txtColor != NULL) 
             && (txtColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,                       txtColor);
                obj_list[obj_idx].clr                                                                     = txtColor;
            }

            if( (bgColor != NULL)
             && (bgColor != obj_list[obj_idx].bgclr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_BGCOLOR,                     bgColor); 
                obj_list[obj_idx].bgclr                                                                   = bgColor;
            }

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,                         ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const int add_data = ArraySize(obj_data);
            const int txt_len = StringLen(text);
            ArrayResize(obj_data, add_data + 1);
            obj_data[add_data].d_width                  = txt_len;
            obj_data[add_data].flags                   |= (font_size << 56); 

            const int new_idx = object_idx(obj_name);
            obj_data[add_data].uid                      = obj_list[new_idx].uid;
            obj_list[new_idx].data_idx                  = add_data;
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = ((angle != NULL) && (angle != EMPTY_VALUE)) ? (uint)(cos(fabs(angle)) * (txt_len * txt_width)) : (txt_len * txt_width);
            obj_list[new_idx].px_height                 = ((angle != NULL) && (angle != EMPTY_VALUE)) ? (uint)(sin(fabs(angle)) * (txt_len * txt_width)) + txt_height : txt_height;
            obj_list[new_idx].clr                       = txtColor;
            obj_list[new_idx].bgclr                     = bgColor;

            ObjectCreate(chart_id, obj_name, OBJ_TEXT, window_id, tm_idx, chart_price);
            
            ObjectSetString(chart_id,   obj_name, OBJPROP_FONT,         "Lucida Sans Typewriter");
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_FONTSIZE,     font_size);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TEXT,         ((text == NULL) ? "\n" : text));
            ObjectSetString(chart_id,   obj_name, OBJPROP_TOOLTIP,      ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_ANCHOR,       (angle > 180) ? ANCHOR_LEFT_UPPER : ANCHOR_LEFT_LOWER);            
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_COLOR,        txtColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BGCOLOR,      bgColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BACK,         false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTABLE,   false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTED,     false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_HIDDEN,       true);
            ObjectSetDouble(chart_id,   obj_name, OBJPROP_ANGLE,        (angle == EMPTY_VALUE) ? NULL : angle);
        }


    // Update font size
    
        _font_size = font_size;

    // Return
    return(obj_name != NULL);
}



//+------------------------------------------------------------------+
//| Draw text label on chart window                                  |
//+------------------------------------------------------------------+
string textlabel(   const uint      x_pos       = NULL, 
                    const uint      y_pos       = NULL, 
                    const string    text        = NULL, 
                    const string    tooltip     = NULL, 
                    const uint      font_size   = 8, 
                    const color     txtColor    = clrWhite, 
                    const color     bgColor     = clrNONE, 
                    const double    angle       = EMPTY_VALUE, 
                    const uint      width       = NULL, 
					const long     	chart_id   	= NULL, 
                    const int       window_id   = NULL)
{
    string tmp_id = NULL; 
    return((textlabel(tmp_id, x_pos, y_pos, text, tooltip, font_size, txtColor, bgColor, angle, width, chart_id, window_id)) ? tmp_id : NULL);
}

bool textlabel( string&         obj_name, 
                const uint      x_pos       = NULL, 
                const uint      y_pos       = NULL, 
                const string    text        = NULL, 
                const string    tooltip     = NULL, 
                const uint      font_size   = 8, 
                const color     txtColor    = clrWhite, 
                const color     bgColor     = clrNONE, 
                const double    angle       = EMPTY_VALUE, 
                const uint      width       = NULL, 
				const long     	chart_id   	= NULL, 
                const int       window_id   = NULL)
{
    // Local init

        static  uint    txt_height  = NULL;
        static  uint    txt_width   = NULL;
        static  uint    _font_size  = NULL;

        if( (_font_size != font_size) 
         && (font_size != NULL) )
         {
            TextSetFont("Lucida Sans Typewriter", font_size * -10, FW_THIN, 0);
            TextGetSize("A", txt_width, txt_height);
        }

        const           int     obj_idx     = draw::object_idx(obj_name);
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_LABEL) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (x_pos == UINT_MAX)
         && (y_pos == UINT_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_LABEL, obj_list[obj_idx].uid, true) == NULL); }
        else if( (x_pos == UINT_MAX)
              || (x_pos == UINT_MAX) )
        { return(false); }
      
      
    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_LABEL, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw textlabel

        // Update existing
        if(obj_idx != -1)
        {
            (x_pos > NULL) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_XDISTANCE,          x_pos);
            
            (y_pos > NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_YDISTANCE,          y_pos);

            if( (font_size != _font_size) 
             || (width != NULL) 
             || (text != NULL) 
             || (angle != EMPTY_VALUE))
            { 
                const uint      _width = ((width == NULL) ? (text != NULL) ? StringLen(text) : obj_data[obj_list[obj_idx].data_idx].d_width : width);
                const double    _angle = (angle == EMPTY_VALUE) ? ObjectGetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_ANGLE) : angle;

                if(font_size != _font_size)
                {
                    ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_FONTSIZE,    font_size); 
                    obj_data[obj_list[obj_idx].data_idx].flags                                = (obj_data[obj_list[obj_idx].data_idx].flags & 0x00FFFFFFFFFFFFFF) | (font_size << 56); 
                }
                
                (width != NULL) &&
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE,           _width * txt_width);

                (font_size != _font_size) &&
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YSIZE,           txt_height);

                (text != NULL) &&
                ObjectSetString(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TEXT,             text); 

                ObjectSetDouble(obj_list[obj_idx].chart_id, obj_name, OBJPROP_ANGLE,            _angle);

                obj_data[obj_list[obj_idx].data_idx].d_width                                  = _width;
                obj_list[obj_idx].px_width                                                    = ((_angle != NULL) && (_angle != EMPTY_VALUE)) ? (uint)(cos(fabs(_angle)) * (_width * txt_width)) : (_width * txt_width);
                obj_list[obj_idx].px_height                                                   = ((_angle != NULL) && (_angle != EMPTY_VALUE)) ? (uint)(sin(fabs(_angle)) * (_width * txt_width)) + txt_height : txt_height;
            }

            if( (txtColor != NULL) 
             && (txtColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,           txtColor);
                obj_list[obj_idx].clr                                                         = txtColor;
            }

            if( (bgColor != NULL)
             && (bgColor != obj_list[obj_idx].bgclr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_BGCOLOR,         bgColor); 
                obj_list[obj_idx].bgclr                                                       = bgColor;
            }

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,             ((tooltip == "") ? "\n" : tooltip));
        }

        // Create new
        else
        {
            const uint _width = ((width == NULL) ? StringLen(text) : width);

            const int add_data = ArraySize(obj_data);
            ArrayResize(obj_data, add_data + 1);
            obj_data[add_data].d_width                  = _width;
            obj_data[add_data].flags                   |= (font_size << 56); 

            const int new_idx = object_idx(obj_name);
            obj_data[add_data].uid                      = obj_list[new_idx].uid;
            obj_list[new_idx].data_idx                  = add_data;
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = ((angle != NULL) && (angle != EMPTY_VALUE)) ? (uint)(cos(fabs(angle)) * (_width * txt_width)) : _width * txt_width;
            obj_list[new_idx].px_height                 = ((angle != NULL) && (angle != EMPTY_VALUE)) ? (uint)(sin(fabs(angle)) * (_width * txt_width) + txt_height) : txt_height;
            obj_list[new_idx].clr                       = txtColor;
            obj_list[new_idx].bgclr                     = bgColor;

            ObjectCreate(chart_id, obj_name, OBJ_LABEL, window_id, NULL, NULL);

            ObjectSetInteger(chart_id,  obj_name, OBJPROP_XDISTANCE,        x_pos);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_YDISTANCE,        y_pos);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_XSIZE,            _width * txt_width);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_YSIZE,            txt_height);
            ObjectSetString(chart_id,   obj_name, OBJPROP_FONT,             "Lucida Sans Typewriter");
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_FONTSIZE,         font_size);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TEXT,             ((text == NULL) ? "\n" : text));
            ObjectSetString(chart_id,   obj_name, OBJPROP_TOOLTIP,          ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_ANCHOR,           (angle > 180) ? ANCHOR_LEFT_UPPER : ANCHOR_LEFT_LOWER);            
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_COLOR,            txtColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BGCOLOR,          bgColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BACK,             false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTABLE,       false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTED,         false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_HIDDEN,           true);
            ObjectSetDouble(chart_id,   obj_name, OBJPROP_ANGLE,            (angle == EMPTY_VALUE) ? NULL : angle);
        }


    // Update font size
    
        _font_size = font_size;

    // Return
    return(obj_name != NULL);
}






/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* Input output object functions                                                                                                                         */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/


//+------------------------------------------------------------------+
//| Draw button on chart window                                      |
//+------------------------------------------------------------------+
bool button_remove(string obj_name)
{ return(button(obj_name, UINT_MAX, UINT_MAX, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)); }

string button(  const uint      x_pos       = NULL, 
                const uint      y_pos       = NULL, 
                const string    text        = NULL, 
                const string    tooltip     = NULL, 
                const uint      font_size   = 8, 
                const color     txtColor    = clrWhite, 
                const color     bgColor     = clrNONE, 
                const uint      width       = NULL, 
				const long     	chart_id   	= NULL, 
                const int       window_id   = NULL)
{
    string tmp_id = NULL; 
    return((button(tmp_id, x_pos, y_pos, text, tooltip, font_size, txtColor, bgColor, width, chart_id, window_id)) ? tmp_id : NULL);
}

bool button(    string&         obj_name, 
                const uint      x_pos       = NULL, 
                const uint      y_pos       = NULL, 
                const string    text        = NULL, 
                const string    tooltip     = NULL, 
                const uint      font_size   = 8, 
                const color     txtColor    = clrWhite, 
                const color     bgColor     = clrDarkGray, 
                const uint      width       = NULL, 
				const long     	chart_id   	= NULL, 
                const int       window_id   = NULL)
{
    // Local init

        static  uint    txt_height  = NULL;
        static  uint    txt_width   = NULL;
        static  uint    _font_size  = NULL;

        if( (_font_size != font_size) 
         && (font_size != NULL) )
         {
            TextSetFont("Lucida Sans Typewriter", font_size * -10, FW_THIN, 0);
            TextGetSize("A", txt_width, txt_height);
        }

        const   static  bool    is_height   = TextGetSize("A", txt_width, txt_height);
        const           int     obj_idx     = draw::object_idx(obj_name);
        

    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_BUTTON) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (x_pos == UINT_MAX)
         && (y_pos == UINT_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_BUTTON, obj_list[obj_idx].uid, true) == NULL); }
        else if( (x_pos == UINT_MAX)
              || (x_pos == UINT_MAX) )
        { return(false); }
      
      
    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_BUTTON, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw button

        // Update existing
        if(obj_idx != -1)
        {
            (x_pos > NULL) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_XDISTANCE,          x_pos);

            (y_pos > NULL) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_YDISTANCE,          y_pos);

            if( (width != NULL) 
             || (font_size != _font_size) )
            { 
                const uint _width = ((width == NULL) ? StringLen((text != NULL) ? text : ObjectGetString(obj_list[obj_idx].chart_id, obj_name, OBJPROP_TEXT)) : width);
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE,           _width * txt_width);
                obj_data[obj_list[obj_idx].data_idx].d_width                                  = _width;
                obj_list[obj_idx].px_width                                                    = _width * txt_width;

                if(font_size != _font_size)
                {
                    ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_FONTSIZE,    font_size); 
                    ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YSIZE,       txt_height);
                    obj_list[obj_idx].px_height                                               = txt_height;
                    obj_data[obj_list[obj_idx].data_idx].flags                                = (obj_data[obj_list[obj_idx].data_idx].flags & 0x00FFFFFFFFFFFFFF) | (font_size << 56); 
                }
            }

            if(text != NULL)
            {
                const uint _width = StringLen(text) * txt_width;
                ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TEXT,            text); 
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE,           _width);
                obj_data[obj_list[obj_idx].data_idx].d_width                                  = StringLen(text);
                obj_list[obj_idx].px_width                                                    = _width;
            }
            
            if( (txtColor != NULL) 
             && (txtColor != obj_list[obj_idx].clr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_COLOR,           txtColor);
                obj_list[obj_idx].clr                                                         = txtColor;
            }

            if( (bgColor != NULL)
             && (bgColor != obj_list[obj_idx].bgclr) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_BGCOLOR,         bgColor); 
                obj_list[obj_idx].bgclr                                                       = bgColor;
            }
            
            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,             ((tooltip == "") ? "\n" : tooltip));

            ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_STATE,               false);
        }

        // Create new
        else
        {
            const int add_data = ArraySize(obj_data);
            const int txt_len = StringLen(text);
            ArrayResize(obj_data, add_data + 1);
            obj_data[add_data].d_width                  = txt_len;
            obj_data[add_data].flags                   |= (font_size << 56); 

            const int new_idx = object_idx(obj_name);
            obj_data[add_data].uid                      = obj_list[new_idx].uid;
            obj_list[new_idx].data_idx                  = add_data;
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = ((width == NULL) ? txt_len * txt_width : width * txt_width);
            obj_list[new_idx].px_height                 = txt_height;
            obj_list[new_idx].clr                       = txtColor;
            obj_list[new_idx].bgclr                     = bgColor;

            ObjectCreate(chart_id, obj_name, OBJ_BUTTON, window_id, NULL, NULL);
            
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_XDISTANCE,        x_pos);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_YDISTANCE,        y_pos);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_XSIZE,            ((width == NULL) ? StringLen(text) : width) * txt_width);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_YSIZE,            txt_height);
            ObjectSetString(chart_id,   obj_name, OBJPROP_FONT,             "Lucida Sans Typewriter");
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_FONTSIZE,         font_size);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TEXT,             ((text == NULL) ? "\n" : text));
            ObjectSetString(chart_id,   obj_name, OBJPROP_TOOLTIP,          ((tooltip == NULL) ? "\n" : tooltip));
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_COLOR,            txtColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BGCOLOR,          bgColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_STATE,            false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BACK,             false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTABLE,       false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTED,         false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_HIDDEN,           true);
        }


    // Update font size
    
        _font_size = font_size;

    // Return
    return(obj_name != NULL);
}


//+------------------------------------------------------------------+
//| Draw edit field on chart window                                  |
//+------------------------------------------------------------------+
bool editfield_remove(string obj_name)
{ return(editfield(obj_name, UINT_MAX, UINT_MAX, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)); }

string editfield(   const uint      x_pos           = NULL, 
                    const uint      y_pos           = NULL, 
                    const uint      width           = NULL, 
                    const string    text            = NULL, 
                    const string    tooltip         = NULL, 
                    const uint      font_size       = 8, 
                    const color     txtColor        = clrWhite, 
                    const color     bgColor         = clrNONE, 
                    const color     borderColor     = clrNONE, 
					const long     	chart_id   		= NULL, 
                    const int       window_id       = NULL)
{
    string tmp_id = NULL; 
    return((editfield(tmp_id, x_pos, y_pos, width, text, tooltip, font_size, txtColor, bgColor, borderColor, chart_id, window_id)) ? tmp_id : NULL);
}

bool editfield( string&         obj_name,
                const uint      x_pos           = NULL, 
                const uint      y_pos           = NULL, 
                const uint      width           = NULL, 
                const string    text            = NULL, 
                const string    tooltip         = NULL, 
                const uint      font_size       = 8, 
                const color     txtColor        = clrWhite, 
                const color     bgColor         = clrNONE, 
                const color     borderColor     = clrNONE, 
				const long     	chart_id   		= NULL, 
                const int       window_id       = NULL)
{
    // Local init
    
        static  uint    txt_height  = NULL;
        static  uint    txt_width   = NULL;
        static  uint    _font_size  = NULL;

        if( (_font_size != font_size) 
         && (font_size != NULL) )
         {
            TextSetFont("Lucida Sans Typewriter", font_size * -10, FW_THIN, 0);
            TextGetSize("A", txt_width, txt_height);
        }

        const   int     obj_idx     = draw::object_idx(obj_name);
                bool    obj_console = false;
                uint    _x_pos      = x_pos;
                uint    _y_pos      = y_pos;
                uint    _width      = width;
                color   _txtColor   = txtColor;
                color   _bgColor    = bgColor;
                color   _brdColor   = borderColor;


    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_EDIT)
         && (object_type(obj_name) != IND_DRAWTYPE_TEXTFIELD) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (!obj_console)
         && (x_pos == UINT_MAX)
         && (y_pos == UINT_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_EDIT, obj_list[obj_idx].uid, true) == NULL); }
        else if( (x_pos == UINT_MAX)
              || (x_pos == UINT_MAX) )
        { return(false); }

      
    // Check for console connection
    
        if( (obj_idx != -1)
         && (object_type(obj_name) == IND_DRAWTYPE_TEXTFIELD) )
        {
            obj_console = true;
            _x_pos      = (uint)ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XDISTANCE);
            _y_pos      = (uint)(ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YDISTANCE) + ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YSIZE));
            _width      = (uint)(ObjectGetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE) / txt_width);
            _txtColor   = obj_list[obj_idx].clr;
            _bgColor    = (color)ChartGetInteger(obj_list[obj_idx].chart_id, CHART_COLOR_BACKGROUND, obj_list[obj_idx].window_id);
            _brdColor   = _bgColor;
        }
    
    
    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_EDIT, ((obj_idx == -1) || obj_console) ? NULL : obj_list[obj_idx].uid);
        if(obj_console) 
        { obj_data[obj_list[obj_idx].data_idx].linked_obj = obj_name; }


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Draw editfield

        // Update existing
        if( (obj_idx != -1) 
         && (!obj_console) )
        { 
            (_x_pos != NULL) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_XDISTANCE,          _x_pos);

            (_y_pos != NULL) && 
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_YDISTANCE,          _y_pos);

            ((_brdColor != NULL) && (_brdColor != clrNONE)) &&
            ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_BORDER_COLOR,       _brdColor);

            if(font_size != _font_size)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_FONTSIZE,       font_size);
                ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_YSIZE,          txt_height);
                obj_list[obj_idx].px_height                                                   = txt_height;
                obj_data[obj_list[obj_idx].data_idx].flags                                    = (obj_data[obj_list[obj_idx].data_idx].flags & 0x00FFFFFFFFFFFFFF) | (font_size << 56); 
            }
            
            if( (width != NULL) 
             || (font_size != NULL) )
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_XSIZE,          _width * txt_width);
                obj_data[obj_list[obj_idx].data_idx].d_width                                  = _width;
                obj_list[obj_idx].px_width                                                    = _width * txt_width;
            }
            
            if(_txtColor != obj_list[obj_idx].clr)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_COLOR,          _txtColor);
                obj_list[obj_idx].clr                                                         = _txtColor;
            }
            
            if(_bgColor != obj_list[obj_idx].bgclr)
            {
                ObjectSetInteger(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_BGCOLOR,        _bgColor);
                obj_list[obj_idx].bgclr                                                       = _bgColor;
            }

            (tooltip != NULL) &&
            ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,             ((tooltip == "") ? "\n" : tooltip));

            ObjectSetString(obj_list[obj_idx].chart_id,   obj_name, OBJPROP_TEXT,               (text == NULL) ? "" : text);
        }
        
        // Create new
        else
        {
            const int add_data = ArraySize(obj_data);
            ArrayResize(obj_data, add_data + 1);
            obj_data[add_data].d_width                  = _width;
            obj_data[add_data].flags                   |= (font_size << 56); 

            const int new_idx = object_idx(obj_name);
            obj_data[add_data].uid                      = obj_list[new_idx].uid;
            obj_list[new_idx].data_idx                  = add_data;
            obj_list[new_idx].chart_id                  = chart_id;
            obj_list[new_idx].window_id                 = window_id;
            obj_list[new_idx].px_width                  = _width * txt_width;
            obj_list[new_idx].px_height                 = txt_height;
            obj_list[new_idx].clr                       = _txtColor;
            obj_list[new_idx].bgclr                     = _bgColor;

            ObjectCreate(chart_id, obj_name, OBJ_EDIT, window_id, NULL, NULL);

            ObjectSetInteger(chart_id,  obj_name, OBJPROP_XDISTANCE,        _x_pos);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_YDISTANCE,        _y_pos);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_XSIZE,            _width * txt_width);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_YSIZE,            txt_height);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TEXT,             (text == NULL) ? "" : text);
            ObjectSetString(chart_id,   obj_name, OBJPROP_TOOLTIP,          (tooltip == NULL) ? "\n" : tooltip);
            ObjectSetString(chart_id,   obj_name, OBJPROP_FONT,             "Lucida Sans Typewriter");
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_FONTSIZE,         font_size);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_ALIGN,            ALIGN_LEFT);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_COLOR,            _txtColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BGCOLOR,          _bgColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BORDER_COLOR,     _brdColor);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_BACK,             false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTABLE,       false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_SELECTED,         false);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_HIDDEN,           true);
            ObjectSetInteger(chart_id,  obj_name, OBJPROP_ZORDER,           LONG_MAX - 5);
        }


    // Update font size
    
        _font_size = font_size;

    // Return
    return(obj_name != NULL);
}


//+------------------------------------------------------------------+
//| Draw textfield on chart window                                   |
//+------------------------------------------------------------------+
bool textfield_remove(string obj_name)
{ return(textfield(obj_name, UINT_MAX, UINT_MAX, NULL, NULL, NULL, NULL, NULL, false, false, NULL, NULL)); }

bool textfield(const string obj_name, const string display_text, const int scroll = NULL)
{
    // Local init
    string out_array[];

    // Split input string to array by line feed
    StringSplit(display_text, 0x0A, out_array);

    // Return
    return(textfield(obj_name, out_array, scroll));
}

bool textfield(const string obj_name, const string& display_text[], const int scroll = NULL)
{
    // Local init
    
        const   static  string      esc_sequence    = CharToString(0x5C) + CharToString(0x1B) + CharToString(0x5B);
        const           int         obj_idx         = draw::object_idx(obj_name);
        const           bool        bottom_up       = obj_data[obj_list[obj_idx].data_idx].flags & 0x01; 
        const           bool        term_colors     = obj_data[obj_list[obj_idx].data_idx].flags & 0x02;
        const           uint        text_lines      = ArraySize(display_text);
                        string      textfield_id    = obj_name;


    // Verify object type

        if( ( (obj_idx != -1)
           && (object_type(obj_name) != IND_DRAWTYPE_TEXTFIELD) )
         || (obj_idx == -1) )
        { return(false); }


    // Prepare output buffer

        ArrayResize(obj_data[obj_list[obj_idx].data_idx].data, obj_data[obj_list[obj_idx].data_idx].d_height, 200);


    // Switch type of buffer loading
    
        if(bottom_up)
        {
            if(term_colors)
            {
                string  result_str  = NULL;
                string  metadata    = NULL;
                int     esc_cnt     = NULL;
                int     esc_begin   = NULL;
                int     esc_end     = NULL;
                int     txt_pos_ptr = NULL;
                int     removed     = NULL;
                int     txt_ptr     = NULL;
                int     line_length = NULL;
                int     lines       = NULL;
                int     last_line   = NULL;
                int     scroll_line = scroll * (scroll < NULL);
                int     out_ptr     = (((int)obj_data[obj_list[obj_idx].data_idx].d_height - 1) - (scroll * (scroll > NULL))) * (scroll < (int)obj_data[obj_list[obj_idx].data_idx].d_height);
                string  esc_cmnd[];
                int     esc_pos[];

                for(int cnt = ((int)text_lines - 1); (cnt > -1) && (out_ptr > -1) && !_StopFlag; cnt--)
                {
                    result_str      = NULL;
                    esc_end         = NULL;
                    esc_cnt         = NULL;
                    ArrayResize(esc_pos, 0, 200);
                    ArrayResize(esc_cmnd, 0, 200);
                    while( (esc_begin != -1) 
                        && (txt_pos_ptr < StringLen(display_text[cnt]))
                        && !_StopFlag )
                    {
                        esc_begin       = StringFind(display_text[cnt], esc_sequence, esc_end);
                        esc_end         = StringFind(display_text[cnt], "m", esc_begin) + 1;
                        
                        result_str     += StringSubstr(display_text[cnt], txt_pos_ptr, (esc_begin - txt_pos_ptr));
                        txt_pos_ptr     = (esc_end * (esc_begin != -1)) + (txt_pos_ptr * (esc_begin == -1));
                        
                        ArrayResize(esc_pos, esc_cnt + 1);
                        ArrayResize(esc_cmnd, esc_cnt + 1);
                        esc_pos[esc_cnt]    = (esc_begin != -1) ? (esc_begin - removed) + 1 : esc_pos[esc_cnt];
                        esc_cmnd[esc_cnt]   = (esc_begin != -1) ? StringSubstr(display_text[cnt], esc_begin, esc_end - esc_begin) : esc_cmnd[esc_cnt];
                        esc_cnt            += (esc_begin != -1);
                        removed            += ((esc_end - esc_begin) * (esc_begin != -1));
                    }
                    esc_cnt--;
                    esc_begin       = NULL;
                    txt_pos_ptr     = NULL;
                    removed         = NULL;

                    line_length     = StringLen(result_str);
                    txt_ptr         = line_length - (line_length % (int)obj_data[obj_list[obj_idx].data_idx].d_width);
                    lines           = (line_length / (int)obj_data[obj_list[obj_idx].data_idx].d_width);
                    last_line       = (out_ptr * (scroll_line > -1)) - ((scroll_line < NULL) || (out_ptr == (int)obj_data[obj_list[obj_idx].data_idx].d_height - 1));
                    while( (txt_ptr > -1)
                        && (out_ptr > -1)
                        && !_StopFlag )
                    {
                        metadata = "";
                        while( (esc_cnt > -1)
                            && (esc_pos[esc_cnt] > txt_ptr)
                            && !_StopFlag )
                        { 
                            metadata = StringFormat("#%i:%s%s", esc_pos[esc_cnt] - txt_ptr, esc_cmnd[esc_cnt], metadata); 
                            esc_cnt--;
                        }

                        obj_data[obj_list[obj_idx].data_idx].data[out_ptr] = StringSubstr(result_str, txt_ptr, obj_data[obj_list[obj_idx].data_idx].d_width) + metadata;
                        lines--;
                        out_ptr -= 1 * (scroll_line >= NULL);
                        scroll_line += (scroll_line < NULL);
                        txt_ptr = lines * (int)obj_data[obj_list[obj_idx].data_idx].d_width;
                    }
                    obj_data[obj_list[obj_idx].data_idx].data[last_line * (last_line != -1)] += (last_line != -1) ? "\n" : NULL;
                }
            }
            else
            {
                int     txt_ptr     = NULL;
                int     line_length = NULL;
                int     lines       = NULL;
                int     last_line   = NULL;
                int     scroll_line = scroll * (scroll < NULL);
                int     out_ptr     = (((int)obj_data[obj_list[obj_idx].data_idx].d_height - 1) - (scroll * (scroll > NULL))) * (scroll < (int)obj_data[obj_list[obj_idx].data_idx].d_height);
                for(int cnt = ((int)text_lines - 1); (cnt > -1) && (out_ptr > -1) && !_StopFlag; cnt--)
                {
                    line_length     = StringLen(display_text[cnt]);
                    txt_ptr         = line_length - (line_length % (int)obj_data[obj_list[obj_idx].data_idx].d_width);
                    lines           = (line_length / (int)obj_data[obj_list[obj_idx].data_idx].d_width);
                    last_line       = (out_ptr * (scroll_line > -1)) - ((scroll_line < NULL) || (out_ptr == (int)obj_data[obj_list[obj_idx].data_idx].d_height - 1));
                    while( (txt_ptr > -1)
                        && (out_ptr > -1)
                        && !_StopFlag )
                    {
                        obj_data[obj_list[obj_idx].data_idx].data[out_ptr] = StringSubstr(display_text[cnt], txt_ptr, obj_data[obj_list[obj_idx].data_idx].d_width);
                        lines--;
                        out_ptr -= 1 * (scroll_line >= NULL);
                        scroll_line += (scroll_line < NULL);
                        txt_ptr = lines * (int)obj_data[obj_list[obj_idx].data_idx].d_width;
                    }
                    obj_data[obj_list[obj_idx].data_idx].data[last_line * (last_line != -1)] += (last_line != -1) ? "\n" : NULL;
                }
            }
        }
        else
        {
            if(term_colors)
            {
                string  result_str  = NULL;
                string  metadata    = NULL;
                int     loop_cnt    = NULL;
                int     esc_cnt     = NULL;
                int     esc_begin   = NULL;
                int     esc_end     = NULL;
                int     txt_pos_ptr = NULL;
                int     removed     = NULL;
                uint    out_ptr     = NULL;
                uint    txt_ptr     = NULL;
                uint    line_length = NULL;
                int     scroll_line = (scroll * (scroll > ((int)obj_data[obj_list[obj_idx].data_idx].d_height * -1))) - (((int)obj_data[obj_list[obj_idx].data_idx].d_height - 1) * (scroll <= ((int)obj_data[obj_list[obj_idx].data_idx].d_height * -1))) + ((int)obj_data[obj_list[obj_idx].data_idx].d_height * (scroll > (int)obj_data[obj_list[obj_idx].data_idx].d_height));
                string  esc_cmnd[];
                int     esc_pos[];

                for(uint cnt = NULL; (cnt < text_lines) && (out_ptr < obj_data[obj_list[obj_idx].data_idx].d_height) && !_StopFlag; cnt++)
                {
                    result_str      = NULL;
                    esc_end         = NULL;
                    esc_cnt         = NULL;
                    loop_cnt        = NULL;
                    ArrayResize(esc_pos, 0, 200);
                    ArrayResize(esc_cmnd, 0, 200);
                    while( (esc_begin != -1) 
                        && (txt_pos_ptr < StringLen(display_text[cnt]))
                        && !_StopFlag )
                    {
                        esc_begin       = StringFind(display_text[cnt], esc_sequence, esc_end);
                        esc_end         = StringFind(display_text[cnt], "m", esc_begin) + 1;
                        
                        result_str     += StringSubstr(display_text[cnt], txt_pos_ptr, (esc_begin - txt_pos_ptr));
                        txt_pos_ptr     = (esc_end * (esc_begin != -1)) + (txt_pos_ptr * (esc_begin == -1));
                        
                        ArrayResize(esc_pos, esc_cnt + 1);
                        ArrayResize(esc_cmnd, esc_cnt + 1);
                        esc_pos[esc_cnt]    = (esc_begin != -1) ? (esc_begin - removed) + 1 : esc_pos[esc_cnt];
                        esc_cmnd[esc_cnt]   = (esc_begin != -1) ? StringSubstr(display_text[cnt], esc_begin, esc_end - esc_begin) : esc_cmnd[esc_cnt];
                        esc_cnt            += (esc_begin != -1);
                        removed            += ((esc_end - esc_begin) * (esc_begin != -1));
                    }
                    esc_begin       = NULL;
                    txt_pos_ptr     = NULL;
                    removed         = NULL;


                    line_length = StringLen(result_str);
                    while( (txt_ptr < line_length)
                        && (out_ptr < obj_data[obj_list[obj_idx].data_idx].d_height)
                        && !_StopFlag )
                    {
                        metadata = "";
                        while( (loop_cnt < esc_cnt)
                            && (esc_pos[esc_cnt] < (int)(txt_ptr + obj_data[obj_list[obj_idx].data_idx].d_width))
                            && !_StopFlag )
                        { 
                            metadata += StringFormat("#%i:%s%s", esc_pos[esc_cnt] - txt_ptr, esc_cmnd[esc_cnt]); 
                            loop_cnt++;
                        }

                        obj_data[obj_list[obj_idx].data_idx].data[out_ptr + ((scroll_line * -1) * (scroll_line < NULL))] = StringSubstr(result_str, txt_ptr, obj_data[obj_list[obj_idx].data_idx].d_width) + metadata;
                        txt_ptr += (obj_data[obj_list[obj_idx].data_idx].d_width) * (scroll_line > -1);
                        out_ptr += (scroll_line < 1);
                        scroll_line -= (scroll_line > NULL);
                        scroll_line += (scroll_line < NULL);
                    }
                    obj_data[obj_list[obj_idx].data_idx].data[out_ptr - (out_ptr > NULL)] += (out_ptr > NULL) ? "\n" : NULL;
                    txt_ptr = NULL;
                }
            }
            else
            {
                uint    out_ptr     = NULL;
                uint    txt_ptr     = NULL;
                uint    line_length = NULL;
                int     scroll_line = (scroll * (scroll > ((int)obj_data[obj_list[obj_idx].data_idx].d_height * -1))) - (((int)obj_data[obj_list[obj_idx].data_idx].d_height - 1) * (scroll <= ((int)obj_data[obj_list[obj_idx].data_idx].d_height * -1))) + ((int)obj_data[obj_list[obj_idx].data_idx].d_height * (scroll > (int)obj_data[obj_list[obj_idx].data_idx].d_height));
                for(uint cnt = NULL; (cnt < text_lines) && (out_ptr < obj_data[obj_list[obj_idx].data_idx].d_height) && !_StopFlag; cnt++)
                {
                    line_length = StringLen(display_text[cnt]);
                    while( (txt_ptr < line_length)
                        && (out_ptr < obj_data[obj_list[obj_idx].data_idx].d_height)
                        && !_StopFlag )
                    {
                        obj_data[obj_list[obj_idx].data_idx].data[out_ptr + ((scroll_line * -1) * (scroll_line < NULL))] = StringSubstr(display_text[cnt], txt_ptr, obj_data[obj_list[obj_idx].data_idx].d_width);
                        txt_ptr += obj_data[obj_list[obj_idx].data_idx].d_width * (scroll_line > -1);
                        out_ptr += (scroll_line < 1);
                        scroll_line -= (scroll_line > NULL);
                        scroll_line += (scroll_line < NULL);
                    }
                    obj_data[obj_list[obj_idx].data_idx].data[out_ptr - (out_ptr > NULL)] += (out_ptr > NULL) ? "\n" : NULL;
                    txt_ptr = NULL;
                }
            }
        }

    // Render textfield and return
    return(textfield(textfield_id, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, bottom_up, term_colors, NULL, NULL, NULL));
}

string textfield(   const uint      x_pos               = NULL, 
                    const uint      y_pos               = NULL, 
                    const uint      textfield_height    = NULL, 
                    const uint      textfield_width     = NULL, 
                    const uint      font_size           = 8, 
                    const color     txtColor            = clrWhite, 
                    const color     bgColor             = clrDimGray, 
                    const uint      transparency        = NULL, 
                    const bool      bottom_up           = false,
                    const bool      term_colors         = false,
                    const string    tooltip             = NULL,
					const long     	chart_id   			= NULL, 
                    const int       window_id           = NULL)
{
    string tmp_id = NULL; 
    return((textfield(tmp_id, x_pos, y_pos, textfield_height, textfield_width, font_size, txtColor, bgColor, transparency, bottom_up, term_colors, tooltip, chart_id, window_id)) ? tmp_id : NULL); 
}

bool textfield( string&         obj_name,
                const uint      x_pos               = NULL, 
                const uint      y_pos               = NULL, 
                const uint      textfield_height    = NULL, 
                const uint      textfield_width     = NULL, 
                const uint      font_size           = 8, 
                const color     txtColor            = clrWhite, 
                const color     bgColor             = clrDimGray, 
                const uint      transparency        = NULL, 
                const bool      bottom_up           = false,
                const bool      term_colors         = false,
                const string    tooltip             = NULL,
				const long     	chart_id   			= NULL, 
                const int       window_id           = NULL)
{
    // Local init
    
        const int obj_idx = draw::object_idx(obj_name);


    // Verify object type

        if( (obj_idx != -1)
         && (object_type(obj_name) != IND_DRAWTYPE_TEXTFIELD) )
        { return(false); }


    // Delete object
    
        if( (obj_idx != -1) 
         && (x_pos == UINT_MAX)
         && (y_pos == UINT_MAX) )
        { return(object_id(chart_id, IND_DRAWTYPE_TEXTFIELD, obj_list[obj_idx].uid, true) == NULL); }
        else if( (x_pos == UINT_MAX)
              || (x_pos == UINT_MAX) )
        { return(false); }
      
      
    // Validate or create

        obj_name = draw::object_id(((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id), IND_DRAWTYPE_TEXTFIELD, (obj_idx == -1) ? NULL : obj_list[obj_idx].uid);


    // Set dynamic object update

        if( (obj_idx > -1)
         && (obj_list[obj_idx].dynamic) )
        { obj_list[obj_idx].chart_redraw = true; }


    // Canvas definition

        static  uint    txt_height  = NULL;
        static  uint    txt_width   = NULL;
        static  uint    _font_size  = NULL;

        if( (_font_size != font_size) 
         && (font_size != NULL) )
         {
            TextSetFont("Lucida Sans Typewriter", font_size * -10, FW_THIN, 0);
            TextGetSize("A", txt_width, txt_height);
            _font_size = font_size;
        }

        const   uint    bg_clr      = (bgColor == NULL) ? obj_list[obj_idx].bgclr : ColorToARGB(bgColor, (uchar)(255.0 - (255.0 * (transparency / 100.0))));
        const   uint    txt_clr     = (txtColor == NULL) ? ColorToARGB(obj_list[obj_idx].clr) : ColorToARGB(txtColor);
        const   uint    trm_xsize   = (uint)(ChartGetInteger((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id, CHART_WIDTH_IN_PIXELS, 0) * 0.95);
        const   uint    trm_ysize   = (uint)(ChartGetInteger((obj_idx == -1) ? chart_id : obj_list[obj_idx].chart_id, CHART_HEIGHT_IN_PIXELS, 0) * 0.95);
        const   uint    _c_width    = (textfield_width == NULL) ? obj_data[obj_list[obj_idx].data_idx].d_width : ((((txt_width * textfield_width) + x_pos) > trm_xsize) ? (trm_xsize / txt_width) : textfield_width);
        const   uint    _c_height   = (textfield_height == NULL) ? obj_data[obj_list[obj_idx].data_idx].d_height : ((((txt_height * textfield_height) + y_pos) > trm_ysize) ? (trm_ysize / txt_height) : textfield_height);
        const   uint    width       = txt_width * _c_width;
        const   uint    height      = txt_height * (_c_height + 1);
        static  uint    textfield_bmp[];

        ArrayResize(textfield_bmp, width * height);
        ArrayInitialize(textfield_bmp, bg_clr);


    // Draw textfield

        // Update existing
        if(obj_idx != -1)
        { 
            // Move console
            
                (x_pos != NULL) && 
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XDISTANCE,       x_pos);
                
                (y_pos != NULL) && 
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YDISTANCE,       y_pos);
                
                (textfield_width != NULL) && 
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_XSIZE,           width);
                
                (textfield_height != NULL) && 
                ObjectSetInteger(obj_list[obj_idx].chart_id, obj_name, OBJPROP_YSIZE,           height);


            // Update metadata
            
                obj_list[obj_idx].clr                               = (color)txt_clr;
                obj_list[obj_idx].bgclr                             = (color)bg_clr;

                if( (obj_data[obj_list[obj_idx].data_idx].d_width  != _c_width)
                 || (obj_data[obj_list[obj_idx].data_idx].d_height != _c_height) )
                {
                    obj_data[obj_list[obj_idx].data_idx].d_width    = _c_width;
                    obj_data[obj_list[obj_idx].data_idx].d_height   = _c_height;
                    obj_list[obj_idx].px_width                      = width;
                    obj_list[obj_idx].px_height                     = height;
                    
                    if(obj_data[obj_list[obj_idx].data_idx].linked_obj != NULL)
                    { editfield(obj_data[obj_list[obj_idx].data_idx].linked_obj, x_pos, y_pos + height, _c_width, NULL, NULL, font_size, txtColor, obj_list[obj_idx].bgclr, clrNONE, chart_id, window_id); }
                }

                (tooltip != NULL) &&
                ObjectSetString(obj_list[obj_idx].chart_id,  obj_name, OBJPROP_TOOLTIP,         ((tooltip == "") ? "\n" : tooltip));


            // Render content

                const int out_ptr = ArraySize(obj_data[obj_list[obj_idx].data_idx].data) - 1;
                if(bottom_up)
                {
                    uint line = _c_height * txt_height;
                    if(term_colors)
                    {
                        uchar txtout_fontsize = (uchar)(obj_data[obj_list[obj_idx].data_idx].flags >> 56);
                        bool eval = false;
                        for(int cnt = (int)out_ptr; (cnt >= (out_ptr - (int)_c_height)) && (cnt > -1) && !_StopFlag; cnt--)
                        { 
                            line -= + (3 * (StringGetCharacter(obj_data[obj_list[obj_idx].data_idx].data[cnt], StringLen(obj_data[obj_list[obj_idx].data_idx].data[cnt]) - 1) == 0x0A));

                            // Check for metadata
                            eval = (StringLen(obj_data[obj_list[obj_idx].data_idx].data[cnt]) > (int)obj_data[obj_list[obj_idx].data_idx].d_width);
                            (eval)  && colors::TextOutColor(obj_data[obj_list[obj_idx].data_idx].data[cnt], obj_data[obj_list[obj_idx].data_idx].d_width, txtout_fontsize, txt_width, NULL, line, TA_LEFT|TA_TOP, textfield_bmp, width, height, txt_clr, bg_clr, COLOR_FORMAT_ARGB_RAW);
                            (!eval) && TextOut(obj_data[obj_list[obj_idx].data_idx].data[cnt], NULL, line, TA_LEFT|TA_TOP, textfield_bmp, width, height, txt_clr, COLOR_FORMAT_ARGB_RAW);
                            line -= txt_height;
                        }
                    }
                    else
                    {
                        for(int cnt = (int)out_ptr; (cnt >= (out_ptr - (int)_c_height)) && (cnt > -1) && !_StopFlag; cnt--)
                        { 
                            line -= + (3 * (StringGetCharacter(obj_data[obj_list[obj_idx].data_idx].data[cnt], StringLen(obj_data[obj_list[obj_idx].data_idx].data[cnt]) - 1) == 0x0A));
                            TextOut(obj_data[obj_list[obj_idx].data_idx].data[cnt], NULL, line, TA_LEFT|TA_TOP, textfield_bmp, width, height, txt_clr, COLOR_FORMAT_ARGB_RAW); 
                            line -= txt_height;
                        }
                    }
                }
                else
                {
                    uint line = NULL;
                    if(term_colors)
                    {
                        uchar txtout_fontsize = (uchar)(obj_data[obj_list[obj_idx].data_idx].flags >> 56);
                        bool eval = false;
                        for(int cnt = (int)out_ptr; (cnt >= (out_ptr - (int)_c_height)) && (cnt > -1) && !_StopFlag; cnt--)
                        { 
                            // Check metadata
                            eval        = (StringLen(obj_data[obj_list[obj_idx].data_idx].data[cnt]) > (int)obj_data[obj_list[obj_idx].data_idx].d_width);
                            (eval)     && colors::TextOutColor(obj_data[obj_list[obj_idx].data_idx].data[cnt], obj_data[obj_list[obj_idx].data_idx].d_width, txtout_fontsize, txt_width, NULL, line, TA_LEFT|TA_TOP, textfield_bmp, width, height, txt_clr, bg_clr, COLOR_FORMAT_ARGB_RAW);
                            (!eval)    && TextOut(obj_data[obj_list[obj_idx].data_idx].data[cnt], NULL, line, TA_LEFT|TA_TOP, textfield_bmp, width, height, txt_clr, COLOR_FORMAT_ARGB_RAW);
                            line       += txt_height + (3 * (StringGetCharacter(obj_data[obj_list[obj_idx].data_idx].data[cnt], StringLen(obj_data[obj_list[obj_idx].data_idx].data[cnt]) - 1) == 0x0A));
                        }
                    }
                    else
                    {
                        for(int cnt = (out_ptr < (int)(_c_height - 1)) ? NULL : out_ptr - (int)(_c_height - 1); (cnt <= out_ptr) && (line < (txt_height * _c_height)) && !_StopFlag; cnt++)
                        { 
                            TextOut(obj_data[obj_list[obj_idx].data_idx].data[cnt], NULL, line, TA_LEFT|TA_TOP, textfield_bmp, width, height, txt_clr, COLOR_FORMAT_ARGB_RAW); 
                            line += txt_height + (3 * (StringGetCharacter(obj_data[obj_list[obj_idx].data_idx].data[cnt], StringLen(obj_data[obj_list[obj_idx].data_idx].data[cnt]) - 1) == 0x0A));
                        }
                    }
                }


            // Update output

                ResourceCreate("::" + obj_name, textfield_bmp, width, height, NULL, NULL, width, COLOR_FORMAT_ARGB_RAW);
                ObjectSetString(obj_list[obj_idx].chart_id, obj_name, OBJPROP_BMPFILE, NULL, "::" + obj_name); 
        }
        
        // Create new
        else
        {
            // Create Canvas resource 

                const int add_data = ArraySize(obj_data);
                const int new_idx = object_idx(obj_name);
                ArrayResize(obj_data, add_data + 1);
                obj_data[add_data].d_width              = _c_width;
                obj_data[add_data].d_height             = _c_height;
                obj_data[add_data].flags               |= (0x01 * bottom_up); 
                obj_data[add_data].flags               |= (0x02 * term_colors); 
                obj_data[add_data].flags               |= (font_size << 56); 
                obj_data[add_data].uid                  = obj_list[new_idx].uid;
                obj_list[new_idx].data_idx              = add_data;
                obj_list[new_idx].chart_id              = chart_id;
                obj_list[new_idx].window_id             = window_id;
                obj_list[new_idx].px_width              = width;
                obj_list[new_idx].px_height             = height;
                obj_list[new_idx].clr                   = txtColor;
                obj_list[new_idx].bgclr                 = (color)bg_clr;

                ResourceFree("::" + obj_name);
                ResourceCreate("::" + obj_name, textfield_bmp, width, height, NULL, NULL, width, COLOR_FORMAT_ARGB_RAW);


            // Create console output field

                ObjectCreate(chart_id, obj_name, OBJ_BITMAP_LABEL, window_id, NULL, NULL);
                
                ObjectSetInteger(chart_id, obj_name, OBJPROP_COLOR,         clrNONE);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_XDISTANCE,     x_pos);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_YDISTANCE,     y_pos);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_XSIZE,         width);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_YSIZE,         height);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_CORNER,        CORNER_LEFT_UPPER);
                ObjectSetString(chart_id,  obj_name, OBJPROP_TOOLTIP,       (tooltip == NULL) ? "\n" : tooltip);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_BACK,          false);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTABLE,    false);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_SELECTED,      false);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_HIDDEN,        true);
                ObjectSetInteger(chart_id, obj_name, OBJPROP_ZORDER,        NULL);
                ObjectSetString(chart_id,  obj_name, OBJPROP_BMPFILE,       NULL, "::" + obj_name);
        }

    // Return
    return(obj_name != NULL);
}


};
// END Draw functions                                                                                                                                     */
//*********************************************************************************************************************************************************/
#endif // LIB_DRAW_MQH_INCLUDED

